package systems.archide.core.controller;

import java.time.LocalDate;
import java.time.Period;
import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import systems.archide.core.pojo.Address;
import systems.archide.core.pojo.BirthCertificateData;
import systems.archide.core.pojo.ChildData;
import systems.archide.core.pojo.DeathCertificateData;
import systems.archide.core.pojo.DeathRecord;
import systems.archide.core.pojo.UnionRecord;
import systems.archide.core.pojo.VitalRecord;
import systems.archide.core.repository.AddressRepository;
import systems.archide.core.repository.DeathRecordRepository;
import systems.archide.core.repository.IdcoreCustomQueries;
import systems.archide.core.repository.UnionRecordRepository;
import systems.archide.core.repository.VitalRecordRepository;

@RestController
@RequestMapping("/death-certificate")
public class DeathCertificateController {
	
	@Autowired
	DeathRecordRepository deathRecordRepository;
	
	@Autowired
	AddressRepository addressRepository;
	
	@Autowired
	VitalRecordRepository vitalRecordRepository;
	
	@Autowired
	UnionRecordRepository unionRecordRepository;
	
	@Autowired
	IdcoreCustomQueries queries;

	@GetMapping("/read/{id}")
	public ResponseEntity<DeathCertificateData> getDeathCertificateData(@PathVariable Long id) {
		DeathRecord deathRecord = deathRecordRepository.findById(id);
		DeathCertificateData data = getDeathCertificateData(deathRecord); 
		return ResponseEntity.ok(data);
	}

	private DeathCertificateData getDeathCertificateData(DeathRecord deathRecord) {
		DeathCertificateData certificateData = new DeathCertificateData();
		certificateData.setId(deathRecord.getId());
	    certificateData.setFirstname(deathRecord.getFirstname());
	    certificateData.setDeathPlace(deathRecord.getDeathLocation());
	    certificateData.setThirdname(deathRecord.getThirdname());
	    certificateData.setFourthname(deathRecord.getFourthname());
	    certificateData.setTimeOfDeath(deathRecord.getTimeOfDeath());
	    certificateData.setDateOfDeath(deathRecord.getDateOfDeath());
		certificateData.setGender(deathRecord.getGender());
		certificateData.setReportedRecord(false);
		certificateData.setVitalRecordId(deathRecord.getVitalRecordId());
		certificateData.setTimestamp(deathRecord.getTimestamp());
	    List<Address> addressList = addressRepository.findByVitalRecordId(deathRecord.getVitalRecordId());
		if (!addressList.isEmpty())
			certificateData.setAddress(addressList.get(0).getAreaId());
		if (deathRecord.getFatherId() != null) {
			VitalRecord fatherRecord = vitalRecordRepository.findById(deathRecord.getFatherId());
			if (fatherRecord != null) {
				certificateData.setFatherName(fatherRecord.getFirstname() + " " + fatherRecord.getFourthname());
			}
		} else if (deathRecord.getReportedFatherName() != null) {
			certificateData.setFatherName(deathRecord.getReportedFatherName());
		}
		if (deathRecord.getMotherId() != null) {
			VitalRecord motherRecord = vitalRecordRepository.findById(deathRecord.getMotherId());
			if (motherRecord != null) {
				certificateData.setMotherName(motherRecord.getFirstname() + " " + motherRecord.getFourthname());
			}
		} else if (deathRecord.getReportedMotherName() != null) {
			certificateData.setMotherName(deathRecord.getReportedMotherName());
		}
	    certificateData.setId(deathRecord.getId());
	    
	    if (deathRecord.getDeclarantId() != null) {
	    	certificateData.setDeclarantId(deathRecord.getDeclarantId());
	    	VitalRecord dr = vitalRecordRepository.findById(deathRecord.getDeclarantId());
	    	if (dr != null) {
	    		certificateData.setDeclarantFirstname(dr.getFirstname());
	    		certificateData.setDeclarantSecondname(dr.getSecondname());
	    		certificateData.setDeclarantFourthname(dr.getFourthname());
	    	}
	    }
	    
	    if (deathRecord.getVitalRecordId() != null) {
	    	VitalRecord r = vitalRecordRepository.findById(deathRecord.getVitalRecordId());
	    	if (r != null) {
	    		certificateData.setBirthdate(r.getBirthdate());
	    		certificateData.setBirthPlace(r.getBirthPlace());
	    	    certificateData.setGender(r.getGender());
	    		List<UnionRecord> uList = null;
	    		if (r.getGender() == 1) {
	    			uList = unionRecordRepository.findByPartner1VitalRecordId(r.getId());
	    			if (!uList.isEmpty())
		    			for (UnionRecord u : uList) {
		    				if (u.getDateOfEffect() == null && u.getDateOfOrder() == null) {
		    					certificateData.setSpouseId(u.getPartner2VitalRecordId());
		    					certificateData.setSpouseName(u.getPartner2Firstname() + " " + u.getPartner2Surname());
		    					certificateData.setMarriageDate(u.getDateOfMarriage());
		    					certificateData.setMarriageLocation(u.getPlace());
		    				}
		    			}
	    			List<VitalRecord> childrenRecods = queries.getFatherChildren(r.getId());
	    			if (!childrenRecods.isEmpty()) {
	    				List<ChildData> children = new ArrayList<>();
	    				for (VitalRecord childRecord : childrenRecods) {
	    					ChildData child = new ChildData();
	    					child.setAge(Period.between(childRecord.getBirthdate(), LocalDate.now()).getYears());
	    					child.setName(childRecord.getFirstname() + " " + childRecord.getFourthname());
	    					child.setGender((childRecord.getGender() == 1)?"MALE":(childRecord.getGender() == 2)?"FEMALE": "Unknown");
	    					children.add(child);
	    				}
	    				certificateData.setChildren(children);
	    			}
	    		}
	    		else if (r.getGender() == 2) {
	    			uList = unionRecordRepository.findByPartner2VitalRecordId(r.getId());
	    			if (!uList.isEmpty())
		    			for (UnionRecord u : uList) {
		    				if (u.getDateOfEffect() == null && u.getDateOfOrder() == null) {
		    					certificateData.setSpouseId(u.getPartner1VitalRecordId());
		    					certificateData.setSpouseName(u.getPartner2Firstname() + " " + u.getPartner1Surname());
		    					certificateData.setMarriageDate(u.getDateOfMarriage());
		    					certificateData.setMarriageLocation(u.getPlace());
		    				}
		    			}
	    			List<VitalRecord> childrenRecods = queries.getMotherChildren(r.getId());
	    			if (!childrenRecods.isEmpty()) {
	    				List<ChildData> children = new ArrayList<>();
	    				for (VitalRecord childRecord : childrenRecods) {
	    					ChildData child = new ChildData();
	    					child.setAge(Period.between(childRecord.getBirthdate(), LocalDate.now()).getYears());
	    					child.setName(childRecord.getFirstname() + " " + childRecord.getFourthname());
	    					child.setGender((childRecord.getGender() == 1)?"MALE":(childRecord.getGender() == 2)?"FEMALE": "Unknown");
	    					children.add(child);
	    				}
	    				certificateData.setChildren(children);
	    			}
	    		}
	    		if (r.getMotherId() != null) {
	    			VitalRecord motherRecord = vitalRecordRepository.findById(r.getMotherId());
	    			if (motherRecord != null)
	    				certificateData.setMotherName(motherRecord.getFirstname() + " " + motherRecord.getFourthname());
	    		} else
    				certificateData.setMotherName(r.getReportedMotherName());
	    		if (r.getFatherId() != null) {
	    			VitalRecord fatherRecord = vitalRecordRepository.findById(r.getFatherId());
	    			if (fatherRecord != null)
	    				certificateData.setFatherName(fatherRecord.getFirstname() + " " + fatherRecord.getFourthname());
	    		} else
    				certificateData.setFatherName(r.getReportedFatherName());
	    	}
	    	else
	    		certificateData.setReportedRecord(true);
	    	
	    }
	    else
	    	certificateData.setReportedRecord(true);
		return certificateData;
	}
}
