package systems.archide.core.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import systems.archide.core.pojo.Address;
import systems.archide.core.pojo.DivorceCertificateResponse;
import systems.archide.core.pojo.UnionRecord;
import systems.archide.core.pojo.VitalRecord;
import systems.archide.core.repository.AddressRepository;
import systems.archide.core.repository.UnionRecordRepository;
import systems.archide.core.repository.VitalRecordRepository;

@RestController
@RequestMapping("/divorce-certificate")
public class DivorceCertificateController {

	@Autowired
	UnionRecordRepository unionRecordRepository;

	@Autowired
	VitalRecordRepository vitalRecordRepository;

	@Autowired
	AddressRepository addressRepository;

	@GetMapping("/read/{id}")
	public ResponseEntity<DivorceCertificateResponse> readDivorceCertificateData(@PathVariable Long id) {
		UnionRecord u = unionRecordRepository.findById(id);
		DivorceCertificateResponse certificate = record2certificate(u);
		return ResponseEntity.ok(certificate);
	}

	private DivorceCertificateResponse record2certificate(UnionRecord u) {
		DivorceCertificateResponse c = new DivorceCertificateResponse();
		c.setDateOfEffect(u.getDateOfEffect());
		c.setDateOfOrder(u.getDateOfOrder());
		c.setDeclarationDate(u.getDateOfMarriage());
		c.setTimeOfMarriage(u.getTimeOfMarriage());
		c.setPartner1Name(u.getPartner1Firstname() + " " + u.getPartner1Surname());
		c.setPartner2Name(u.getPartner2Firstname() + " " + u.getPartner2Surname());
		c.setDeclarationPlace(u.getPlace());
		c.setAnnulmentOrder(u.getAnnulmentOrder());
		c.setUnionRecordId(u.getId());
		if (u.getPartner1VitalRecordId() != null) {
			VitalRecord p1Record = vitalRecordRepository.findById(u.getPartner1VitalRecordId());
			if (p1Record != null) {
				c.setPartner1Name(p1Record.getFirstname() + " " + p1Record.getFourthname());
			}
			if (u.getPartner2VitalRecordId() != null) {
				VitalRecord p2Record = vitalRecordRepository.findById(u.getPartner2VitalRecordId());
				if (p2Record != null) {
					c.setPartner2Name(p2Record.getFirstname() + " " + p2Record.getFourthname());
				}
			}
		}
		return c;
	}

}
