package systems.archide.core.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import systems.archide.core.pojo.Address;
import systems.archide.core.pojo.MarriageCertificateResponse;
import systems.archide.core.pojo.UnionRecord;
import systems.archide.core.pojo.VitalRecord;
import systems.archide.core.repository.AddressRepository;
import systems.archide.core.repository.UnionRecordRepository;
import systems.archide.core.repository.VitalRecordRepository;

@RestController
@RequestMapping("/marriage-certificate")
public class MarriageCertificateController {

	@Autowired
	UnionRecordRepository unionRecordRepository;

	@Autowired
	VitalRecordRepository vitalRecordRepository;

	@Autowired
	AddressRepository addressRepository;

	@GetMapping("/read/{id}")
	public ResponseEntity<MarriageCertificateResponse> readMarriageCertificateData(@PathVariable Long id) {
		UnionRecord u = unionRecordRepository.findById(id);
		MarriageCertificateResponse certificate = record2certificate(u);
		return ResponseEntity.ok(certificate);
	}

	private MarriageCertificateResponse record2certificate(UnionRecord u) {
		MarriageCertificateResponse c = new MarriageCertificateResponse();
		c.setDeclarationDate(u.getDateOfMarriage());
		c.setPartner1VitalRecordId(u.getPartner1VitalRecordId());
		c.setPartner2VitalRecordId(u.getPartner2VitalRecordId());
		c.setPartner1reported(false);
		c.setPartner2reported(false);
		if (u.getCelebrantId() != null) {
			VitalRecord cr = vitalRecordRepository.findById(u.getCelebrantId());
			if (cr != null)
				c.setCelebrant(cr.getFourthname() + " " + cr.getFirstname() );
		} else
			c.setCelebrant(u.getReportedCelebrant());
		if (u.getOfficerId() != null) {
			VitalRecord or = vitalRecordRepository.findById(u.getOfficerId());
			if (or != null)
				c.setOfficer(or.getFourthname() + " " + or.getFirstname() );
		} else
			c.setOfficer(u.getReportedOfficer());
		
		if (u.getDeclarantId() != null) {
			VitalRecord dr = vitalRecordRepository.findById(u.getDeclarantId());
			if (dr != null)
				c.setDeclarationFullName(dr.getFourthname() + " " + dr.getFirstname() );
		} else
			c.setDeclarationFullName(u.getReportedDeclarant());
		c.setPosition(u.getPosition());
		c.setMarriageType(u.getUnionRegime());
		c.setTimeOfMarriage(u.getTimeOfMarriage());
		c.setDeclarationPlace(u.getAddress());
		c.setRegistrationLocation(u.getRegistrationLocation());
		c.setMentions(u.getMentions());
		if (u.getFirstWitnessId() != null) {
			c.setFirstWitnessId(u.getFirstWitnessId());
			VitalRecord fw = vitalRecordRepository.findById(u.getFirstWitnessId());
			if (fw != null)
				c.setFirstWitnessName(fw.getFirstname() + " " + fw.getFourthname());
		}
		if (u.getSecondWitnessId() != null) {
			c.setSecondWitnessId(u.getSecondWitnessId());
			VitalRecord fw = vitalRecordRepository.findById(u.getSecondWitnessId());
			if (fw != null)
				c.setSecondWitnessName(fw.getFirstname() + " " + fw.getFourthname());
		}
		if (u.getPartner1VitalRecordId() != null) {
			VitalRecord p1Record = vitalRecordRepository.findById(u.getPartner1VitalRecordId());
			if (p1Record != null) {
				c.setPartner1BirthLocation(p1Record.getBirthPlace());
				c.setPartner1DateOfBirth(p1Record.getBirthdate());
				c.setPartner1FirstNames(p1Record.getFirstname());
				c.setPartner1MelanesianName(p1Record.getThirdname());
				c.setPartner1Surname(p1Record.getFourthname());
				c.setPartner1Sex(p1Record.getGender());
				List<Address> addressList = addressRepository.findByVitalRecordId(p1Record.getId());
				if (!addressList.isEmpty())
					c.setPartner1UsualResidence(addressList.get(0).getAreaId());
				if (p1Record.getFatherId() != null) {
					VitalRecord p1FatherRecord = vitalRecordRepository.findById(p1Record.getFatherId());
					if (p1FatherRecord != null) {
						c.setPartner1FatherName(p1FatherRecord.getFirstname() + " " + p1FatherRecord.getFourthname());
					}
				} else if (p1Record.getReportedFatherName() != null) {
					c.setPartner1FatherName(p1Record.getReportedFatherName());
				}
				if (p1Record.getMotherId() != null) {
					VitalRecord p1MotherRecord = vitalRecordRepository.findById(p1Record.getMotherId());
					if (p1MotherRecord != null) {
						c.setPartner1MotherName(p1MotherRecord.getFirstname() + " " + p1MotherRecord.getFourthname());
					}
				} else if (p1Record.getReportedMotherName() != null) {
					c.setPartner1MotherName(p1Record.getReportedMotherName());
				}
			}
			else {
				c.setPartner1FirstNames(u.getPartner1Firstname());
				c.setPartner1Surname(u.getPartner1Surname());
				c.setPartner1FatherName(u.getPartner1FatherName());
				c.setPartner1MotherName(u.getPartner1MotherName());
				c.setPartner1reported(true);
			}
		}
		if (u.getPartner2VitalRecordId() != null) {
			VitalRecord p2Record = vitalRecordRepository.findById(u.getPartner2VitalRecordId());
			if (p2Record != null) {
				c.setPartner2BirthLocation(p2Record.getBirthPlace());
				c.setPartner2DateOfBirth(p2Record.getBirthdate());
				c.setPartner2FirstNames(p2Record.getFirstname());
				c.setPartner2MelanesianName(p2Record.getThirdname());
				c.setPartner2Surname(p2Record.getFourthname());
				c.setPartner2Sex(p2Record.getGender());
				List<Address> addressList = addressRepository.findByVitalRecordId(p2Record.getId());
				if (!addressList.isEmpty())
					c.setPartner2UsualResidence(addressList.get(0).getAreaId());
				if (p2Record.getFatherId() != null) {
					VitalRecord p2FatherRecord = vitalRecordRepository.findById(p2Record.getFatherId());
					if (p2FatherRecord != null) {
						c.setPartner2FatherName(p2FatherRecord.getFirstname() + " " + p2FatherRecord.getFourthname());
					}
				} else if (p2Record.getReportedFatherName() != null) {
					c.setPartner2FatherName(p2Record.getReportedFatherName());
				}
				if (p2Record.getMotherId() != null) {
					VitalRecord p2MotherRecord = vitalRecordRepository.findById(p2Record.getMotherId());
					if (p2MotherRecord != null) {
						c.setPartner2MotherName(p2MotherRecord.getFirstname() + " " + p2MotherRecord.getFourthname());
					}
				} else if (p2Record.getReportedMotherName() != null) {
					c.setPartner2MotherName(p2Record.getReportedMotherName());

				}
			} else {
				c.setPartner2FirstNames(u.getPartner2Firstname());
				c.setPartner2Surname(u.getPartner2Surname());
				c.setPartner2FatherName(u.getPartner2FatherName());
				c.setPartner2MotherName(u.getPartner2MotherName());
				c.setPartner2reported(true);
			}	
		}
		return c;
	}

}
