package systems.archide.core.repository;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Repository;

import com.softomaton.orm.IStore;
import com.softomaton.orm.QueryUtil;
import com.softomaton.orm.UserContext;

import systems.archide.core.pojo.PreparedQuery;
import systems.archide.core.pojo.QueryBuilder;
import systems.archide.core.pojo.SearchQueryInput;
import systems.archide.core.pojo.VitalRecord;

@Lazy
@Repository
public class VitalRecordAdvancedSearchRepository {
	private static final Logger log = LogManager.getLogger(systems.archide.core.repository.VitalRecordAdvancedSearchRepository.class);

	IStore store;
	UserContext userContext;

	public void setUserContext(UserContext userContext) {
		this.userContext = userContext;
	}

	public UserContext getUserContext() {
		return this.userContext;
	}

	public void setStore(IStore store) {
		this.store = store;
	}

	public IStore getStore() {
		return store;
	}
	
	public List<VitalRecord> advancedSearch(SearchQueryInput input, Long limit, Long offset, String orderBy) {
		List<VitalRecord> beans = new ArrayList<VitalRecord>();
		Connection con = store.getConnection();
		QueryBuilder builder = new QueryBuilder();
		PreparedQuery preparedQuery = builder.buildQuery(input, "vital_record", limit, offset, orderBy);
		PreparedStatement ps = null;
		if (con != null) {
			try {
				ps = con.prepareStatement(preparedQuery.getQuery());
				for (int i = 0; i < preparedQuery.getParameters().size(); i++) {
				    ps.setObject(i + 1, preparedQuery.getParameters().get(i));
				}
				ps.execute();
				ResultSet rs = ps.getResultSet();
				while (rs.next()) {
					VitalRecord bean = new VitalRecord();
					bean.setId((Long) rs.getObject("id"));
					bean.setBirthCountry((Integer) rs.getObject("birth_country"));
					bean.setBirthDayUnknown(rs.getBoolean("birth_day_unknown"));
					bean.setBirthMonthUnknown(rs.getBoolean("birth_month_unknown"));
					bean.setBirthPlace(rs.getString("birth_place"));
					bean.setBirthTime(rs.getObject("birth_time", LocalTime.class));
					bean.setBirthTimeUnknown(rs.getBoolean("birth_time_unknown"));
					bean.setBirthYearUnknown(rs.getBoolean("birth_year_unknown"));
					bean.setBirthdate(rs.getObject("birthdate", LocalDate.class));
					bean.setCardId((Long) rs.getObject("card_id"));
					bean.setCommonIdNumber(rs.getString("common_id_number"));
					bean.setDisability(rs.getBoolean("disability"));
					bean.setDonor(rs.getBoolean("donor"));
					bean.setDonorNotes(rs.getString("donor_notes"));
					bean.setDraft(rs.getBoolean("draft"));
					bean.setFaceId((Long) rs.getObject("face_id"));
					bean.setFatherId((Long) rs.getObject("father_id"));
					bean.setFifthname(rs.getString("fifthname"));
					bean.setFirstname(rs.getString("firstname"));
					bean.setFourthname(rs.getString("fourthname"));
					bean.setGender(QueryUtil.getNullableShort(rs, "gender"));
					bean.setInactive(rs.getBoolean("inactive"));
					bean.setIsDeleted(rs.getBoolean("is_deleted"));
					bean.setMaritalStatus(QueryUtil.getNullableShort(rs, "marital_status"));
					bean.setMarkedInactive(rs.getObject("marked_inactive", LocalDate.class));
					bean.setMotherId((Long) rs.getObject("mother_id"));
					bean.setNotes(rs.getString("notes"));
					bean.setOtherCitizenshipCsv(rs.getString("other_citizenship_csv"));
					bean.setPrimaryCitizenship(rs.getString("primary_citizenship"));
					bean.setReportedFatherName(rs.getString("reported_father_name"));
					bean.setReportedMotherName(rs.getString("reported_mother_name"));
					bean.setSecondname(rs.getString("secondname"));
					bean.setThirdname(rs.getString("thirdname"));
					bean.setTimestamp(rs.getObject("timestamp", LocalDateTime.class));
					bean.setTouched((Long) rs.getObject("touched"));
					bean.setUnknownFather(rs.getBoolean("unknown_father"));
					bean.setVoter(rs.getBoolean("voter"));
					beans.add(bean);
				}
			} catch (SQLException e) {
				log.error("find where failed", e);
				e.printStackTrace();
			} finally {
				if (ps != null)
					try {
						ps.close();
					} catch (SQLException e) {

						log.error("error while closing prepared statement", e);
						throw new RuntimeException(e);
					}
				if (con != null) {
					try {
						con.close();
					} catch (SQLException e) {
						log.error("error while closing connection", e);
						throw new RuntimeException(e);
					}
				}
			}
		}
		return beans;
	}

	public long advancedSearchCount(SearchQueryInput input) {
		Connection con = store.getConnection();
		QueryBuilder builder = new QueryBuilder();
		PreparedQuery preparedQuery = builder.buildQuery(input, "vital_record");
		PreparedStatement ps = null;
		if (con != null) {
			try {
				ps = con.prepareStatement(preparedQuery.getQuery());
				for (int i = 0; i < preparedQuery.getParameters().size(); i++) {
				    ps.setObject(i + 1, preparedQuery.getParameters().get(i));
				}
				ps.execute();
				ResultSet rs = ps.getResultSet();
				if (rs.next()) {
					return rs.getLong(1);
				}
			} catch (SQLException e) {
				log.error("find where failed", e);
				e.printStackTrace();
			} finally {
				if (ps != null)
					try {
						ps.close();
					} catch (SQLException e) {

						log.error("error while closing prepared statement", e);
						throw new RuntimeException(e);
					}
				if (con != null) {
					try {
						con.close();
					} catch (SQLException e) {
						log.error("error while closing connection", e);
						throw new RuntimeException(e);
					}
				}
			}
		}
		return -1;
	}

}
