package systems.archide.core.service.hooks;

import java.time.LocalDate;
import java.util.List;

import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;

import systems.archide.core.pojo.AttachDigitalizationMtlbInput;
import systems.archide.core.pojo.CivilStatusMtlb;
import systems.archide.core.pojo.CivilStatusMtlbAttachment;
import systems.archide.core.pojo.CivilStatusMtlbCreateRequest;
import systems.archide.core.pojo.DigitalizationCreateRequest;
import systems.archide.core.pojo.DocumentType;
import systems.archide.core.pojo.Face;
import systems.archide.core.pojo.GetServiceStatusRequest;
import systems.archide.core.pojo.GetServiceStatusResponse;
import systems.archide.core.pojo.IdMtlb;
import systems.archide.core.pojo.IdMtlbCreateRequest;
import systems.archide.core.pojo.IssueDocumentInput;
import systems.archide.core.pojo.IssuedDocuments;
import systems.archide.core.pojo.MtlbStatus;
import systems.archide.core.pojo.MtlbType;
import systems.archide.core.pojo.Rv4DeathMtlbCreateRequest;
import systems.archide.core.pojo.Rv4DivorceMtlbRequest;
import systems.archide.core.pojo.Rv4IdCardMtlbRequest;
import systems.archide.core.pojo.Rv4MarriageMtlbRequest;
import systems.archide.core.pojo.RvFourBirthMtlbCreateRequest;
import systems.archide.core.pojo.RvfourBirthMtlb;
import systems.archide.core.pojo.RvfourDeathMtlb;
import systems.archide.core.pojo.RvfourDivorceMtlb;
import systems.archide.core.pojo.RvfourIdCardMtlb;
import systems.archide.core.pojo.RvfourMarriageMtlb;
import systems.archide.core.pojo.UnionMtlb;
import systems.archide.core.pojo.VitalRecord;
import systems.archide.core.repository.CivilStatusMtlbAttachmentRepository;
import systems.archide.core.repository.CivilStatusMtlbRepository;
import systems.archide.core.repository.FaceRepository;
import systems.archide.core.repository.IdMtlbRepository;
import systems.archide.core.repository.IssuedDocumentsRepository;
import systems.archide.core.repository.RvfourBirthMtlbRepository;
import systems.archide.core.repository.RvfourDeathMtlbRepository;
import systems.archide.core.repository.RvfourDivorceMtlbRepository;
import systems.archide.core.repository.RvfourIdCardMtlbRepository;
import systems.archide.core.repository.RvfourMarriageMtlbRepository;
import systems.archide.core.repository.UnionMtlbRepository;
import systems.archide.core.repository.VitalRecordRepository;
import systems.archide.core.workflows.utils.MtlbEntity;

@Component
public class ApplyLogic {
	
	final Integer EXPIRES_YEARS = 5;

	@Autowired
	private CivilStatusMtlbRepository civilStatusMtlbService;

	@Autowired
	private CivilStatusMtlbAttachmentRepository civilStatusMtlbAttachmentService;

	@Autowired
	private IdMtlbRepository idMtlbService;

	@Autowired
	private UnionMtlbRepository unionMtlbService;

	@Autowired
	private RvfourBirthMtlbRepository rvfourBirthMtlbRepository;

	@Autowired
	private RvfourDeathMtlbRepository rvfourDeathMtlbRepository;

	@Autowired
	private VitalRecordRepository vitalRecordService;
	
	@Autowired
	private RvfourMarriageMtlbRepository rvfourMarriageMtlbRepository;
	
	@Autowired
	private RvfourDivorceMtlbRepository rvfourDivorceMtlbRepository;
	
	@Autowired
	private RvfourIdCardMtlbRepository rvfourIdCardMtlbRepository;

	@Autowired
	private FaceRepository faceService;
	
	@Autowired
	private IssuedDocumentsRepository issuedDocumentsRepository;

	public final String INITIATED = "INITIATED";

	public final String FINISHED = "FINISHED";
	
	public final Integer CIVIL_STATUS_MTLB_TYPE = 1;
	
	public final Integer ID_MTLB_TYPE = 2;

	public ResponseEntity<Long> createCivilStatusMtlb(CivilStatusMtlbCreateRequest request) {
		CivilStatusMtlb civilStatusMtlb = civilStatusMtlbCreateRequest2CivilStatusMtlb(request);
		civilStatusMtlb.setMtlbStatus(MtlbStatus.INITIATED);
		Long id = civilStatusMtlbService.insert(civilStatusMtlb);
		civilStatusMtlb.setId(id);
		return ResponseEntity.ok(id);
	}

	public ResponseEntity<Long> createIdMtlb(IdMtlbCreateRequest request) {
		IdMtlb idMtlb = idMtlbCreateRequest2IdMtlb(request);
		idMtlb.setMtlbStatus(MtlbStatus.INITIATED);
		Long id = idMtlbService.insert(idMtlb);
		return ResponseEntity.ok(id);
	}

	public UnionMtlb createUnionMtlb(UnionMtlb request) {
		request.setMtlbStatus(MtlbStatus.SUBMITTED);
		Long id = unionMtlbService.insert(request);
		request.setId(id);
		return request;
	}

	private CivilStatusMtlb civilStatusMtlbCreateRequest2CivilStatusMtlb(
			CivilStatusMtlbCreateRequest civilStatusMtlbCreateRequest) {
		CivilStatusMtlb civilStatusMtlb = new CivilStatusMtlb();
		civilStatusMtlb.setDraft(civilStatusMtlbCreateRequest.isDraft());
		civilStatusMtlb.setFirstname(civilStatusMtlbCreateRequest.getFirstname());
		civilStatusMtlb.setSecondname(civilStatusMtlbCreateRequest.getSecondname());
		civilStatusMtlb.setThirdname(civilStatusMtlbCreateRequest.getThirdname());
		civilStatusMtlb.setFourthname(civilStatusMtlbCreateRequest.getFourthname());
		civilStatusMtlb.setFifthname(civilStatusMtlbCreateRequest.getFifthname());
		civilStatusMtlb.setMtlbStatus(MtlbStatus.INITIATED);
		civilStatusMtlb.setMtlbType(MtlbType.valueOf(civilStatusMtlbCreateRequest.getMtlbType()));
		civilStatusMtlb.setBirthdate(civilStatusMtlbCreateRequest.getBirthdate());
		civilStatusMtlb.setGender(civilStatusMtlbCreateRequest.getGender());
		civilStatusMtlb.setFace(civilStatusMtlbCreateRequest.getFace());
		civilStatusMtlb.setFaceMimeType(civilStatusMtlbCreateRequest.getFaceMimeType());
		civilStatusMtlb.setVitalRecordId(civilStatusMtlbCreateRequest.getVitalRecordId());
		civilStatusMtlb.setBirthPlace(civilStatusMtlbCreateRequest.getBirthPlace());
		civilStatusMtlb.setReportedFatherName(civilStatusMtlbCreateRequest.getReportedFatherName());
		civilStatusMtlb.setReportedMotherName(civilStatusMtlbCreateRequest.getReportedMotherName());
		civilStatusMtlb.setFatherId(civilStatusMtlbCreateRequest.getFatherId());
		civilStatusMtlb.setMotherId(civilStatusMtlbCreateRequest.getMotherId());
		civilStatusMtlb.setDeclarantId(civilStatusMtlbCreateRequest.getDeclarantId());
		civilStatusMtlb.setNotes(civilStatusMtlbCreateRequest.getNotes());
		civilStatusMtlb.setMaritalStatus(civilStatusMtlbCreateRequest.getMaritalStatus());
		return civilStatusMtlb;
	}

	private IdMtlb idMtlbCreateRequest2IdMtlb(IdMtlbCreateRequest idMtlbCreateRequest) {
		IdMtlb idMtlb = new IdMtlb();
		idMtlb.setDraft(idMtlbCreateRequest.isDraft());
		idMtlb.setFace(idMtlbCreateRequest.getFace());
		idMtlb.setFaceMimeType(idMtlbCreateRequest.getFaceMimeType());
		idMtlb.setVitalRecordId(idMtlbCreateRequest.getVitalRecordId());
		idMtlb.setMtlbType(MtlbType.valueOf(idMtlbCreateRequest.getMtlbType()));
		return idMtlb;
	}

	public ResponseEntity<Long> createDigitalizationMtlb(DigitalizationCreateRequest digitalizationCreateRequest) {
		CivilStatusMtlb mtlb = new CivilStatusMtlb();
		CivilStatusMtlbAttachment mtlbAttachment = new CivilStatusMtlbAttachment();
		mtlb.setDraft(false);
		mtlb.setMtlbStatus(MtlbStatus.NOT_ASSIGNED);
		mtlb.setMtlbType(MtlbType.DIGITALIZATION);
		Long mtlbId = civilStatusMtlbService.insert(mtlb);
		mtlbAttachment.setCivilStatusMtlbId(mtlbId);
		mtlbAttachment.setFileName(digitalizationCreateRequest.getFileName());
		mtlbAttachment.setContent(digitalizationCreateRequest.getContent());
		civilStatusMtlbAttachmentService.insert(mtlbAttachment);
		return ResponseEntity.ok(mtlbId);
	}

	public ResponseEntity<Long> attachDigitalizationMtlb(AttachDigitalizationMtlbInput attachDigitalizationMtlbInput) {
		CivilStatusMtlb civilStatusMtlb = civilStatusMtlbService
				.findById(attachDigitalizationMtlbInput.getCivilStatusMtlbId());
		VitalRecord vr = vitalRecordService.findById(attachDigitalizationMtlbInput.getVitalRecordId());
		Long id = null;
		if (civilStatusMtlb != null && vr != null) {
			vitalRecord2CivilStatusMtlb(civilStatusMtlb, vr);
			civilStatusMtlbService.update(civilStatusMtlb);
			id = civilStatusMtlb.getId();
		}
		return ResponseEntity.ok(id);
	}

	private void vitalRecord2CivilStatusMtlb(CivilStatusMtlb civilStatusMtlb, VitalRecord vr) {
		civilStatusMtlb.setFirstname(vr.getFirstname());
		civilStatusMtlb.setSecondname(vr.getSecondname());
		civilStatusMtlb.setThirdname(vr.getThirdname());
		civilStatusMtlb.setFourthname(vr.getFourthname());
		civilStatusMtlb.setFifthname(vr.getFifthname());
		civilStatusMtlb.setBirthdate(vr.getBirthdate());
		civilStatusMtlb.setMtlbStatus(MtlbStatus.READY_FOR_APPROVAL);
		civilStatusMtlb.setMtlbType(MtlbType.CIVIL_STATUS_CHANGE);
		civilStatusMtlb.setGender(vr.getGender());
		if (vr.getFaceId() != null) {
			Face face = faceService.findById(vr.getFaceId());
			civilStatusMtlb.setFace(face.getImage());
		}
		civilStatusMtlb.setFaceMimeType("image/png");
		civilStatusMtlb.setBirthTime(vr.getBirthTime());
		civilStatusMtlb.setNotes(vr.getNotes());
		civilStatusMtlb.setReportedFatherName(vr.getReportedFatherName());
		civilStatusMtlb.setFatherId(vr.getFatherId());
		civilStatusMtlb.setMotherId(vr.getMotherId());
		civilStatusMtlb.setReportedMotherName(vr.getReportedMotherName());
		civilStatusMtlb.setVitalRecordId(vr.getId());
	}

	private RvfourBirthMtlb rvFourBirthMtlbCreateRequest2RvfourBirthMtlb(
			RvFourBirthMtlbCreateRequest rvFourBirthMtlbCreateRequest) {
		RvfourBirthMtlb mtlb = new RvfourBirthMtlb();
		mtlb.setAddress(rvFourBirthMtlbCreateRequest.getAddress());
		mtlb.setBirthdate(rvFourBirthMtlbCreateRequest.getBirthdate());
		mtlb.setBirthDayUnknown(rvFourBirthMtlbCreateRequest.isBirthDayUnknown());
		mtlb.setBirthMonthUnknown(rvFourBirthMtlbCreateRequest.isBirthMonthUnknown());
		mtlb.setBirthPlace(rvFourBirthMtlbCreateRequest.getBirthPlace());
		mtlb.setBirthYearUnknown(rvFourBirthMtlbCreateRequest.isBirthYearUnknown());
		mtlb.setDraft(false);
		mtlb.setEmail(rvFourBirthMtlbCreateRequest.getEmail());
		mtlb.setFace(rvFourBirthMtlbCreateRequest.getFace());
		mtlb.setMtlbStatus(MtlbStatus.INITIATED);
		mtlb.setTimestamp(rvFourBirthMtlbCreateRequest.getTimestamp());
		mtlb.setMtlbType(MtlbType.valueOf(rvFourBirthMtlbCreateRequest.getType()));
		mtlb.setFatherId(rvFourBirthMtlbCreateRequest.getFatherId());
		mtlb.setMotherId(rvFourBirthMtlbCreateRequest.getMotherId());
		mtlb.setReportedFatherName(rvFourBirthMtlbCreateRequest.getReportedFatherName());
		mtlb.setReportedMotherName(rvFourBirthMtlbCreateRequest.getReportedMotherName());
		mtlb.setFirstname(rvFourBirthMtlbCreateRequest.getFirstname());
		mtlb.setSecondname(rvFourBirthMtlbCreateRequest.getSecondname());
		mtlb.setThirdname(rvFourBirthMtlbCreateRequest.getThirdname());
		mtlb.setFourthname(rvFourBirthMtlbCreateRequest.getFourthname());
		mtlb.setFifthname(rvFourBirthMtlbCreateRequest.getFifthname());
		mtlb.setSixthname(rvFourBirthMtlbCreateRequest.getSixthname());
		mtlb.setVitalRecordId(rvFourBirthMtlbCreateRequest.getVitalRecordId());
		if (rvFourBirthMtlbCreateRequest.getBirthtime() != null)
			mtlb.setBirthtime(rvFourBirthMtlbCreateRequest.getBirthtime().toLocalTime());
		mtlb.setMultiBirth(rvFourBirthMtlbCreateRequest.isMultiBirth());
		mtlb.setOldAddress(rvFourBirthMtlbCreateRequest.getOldAddress());
		mtlb.setReligion(rvFourBirthMtlbCreateRequest.getReligion());
		mtlb.setOccupation(rvFourBirthMtlbCreateRequest.getOccupation());
		mtlb.setGender(rvFourBirthMtlbCreateRequest.getGender());
		mtlb.setAnnex(rvFourBirthMtlbCreateRequest.getAnnex());
		mtlb.setNumberOfAttachments(rvFourBirthMtlbCreateRequest.getNumberOfAttachments());
		return mtlb;
	}

	public ResponseEntity<Long> saveRv4BirthMtlbByVitalRecordId(RvFourBirthMtlbCreateRequest rvFourBirthMtlbCreateRequest) {
		RvfourBirthMtlb mtlb = rvfourBirthMtlbRepository
				.findByVitalRecordId(rvFourBirthMtlbCreateRequest.getVitalRecordId());
		if (mtlb != null && !MtlbType.valueOf(rvFourBirthMtlbCreateRequest.getType()).equals(MtlbType.RV4_VITAL_RECORD_HISTORY))
			return ResponseEntity.ok(mtlb.getId());
		else {
			RvfourBirthMtlb newMtlb = rvFourBirthMtlbCreateRequest2RvfourBirthMtlb(rvFourBirthMtlbCreateRequest);
			Long id = rvfourBirthMtlbRepository.insert(newMtlb);
			return ResponseEntity.ok(id);
		}
	}

	public ResponseEntity<Long> saveRv4DeathMtlbByVitalRecordId(Rv4DeathMtlbCreateRequest rv4DeathMtlbCreateRequest) {
		RvfourDeathMtlb mtlb = rvfourDeathMtlbRepository
				.findByVitalRecordId(rv4DeathMtlbCreateRequest.getVitalRecordId());
		if (mtlb != null)
			return ResponseEntity.ok(mtlb.getId());
		else {
			RvfourDeathMtlb newMtlb = rvFourDeathMtlbCreateRequest2RvfourDeathMtlb(rv4DeathMtlbCreateRequest);
			return ResponseEntity.ok(rvfourDeathMtlbRepository.insert(newMtlb));
		}
	}

	private RvfourDeathMtlb rvFourDeathMtlbCreateRequest2RvfourDeathMtlb(
			Rv4DeathMtlbCreateRequest request) {
		RvfourDeathMtlb	mtlb = new RvfourDeathMtlb();
		mtlb.setAnnex(request.getAnnex());
		mtlb.setDateOfDeath(request.getDateOfDeath());
		mtlb.setDeclarantId(request.getDeclarantId());
		mtlb.setVitalRecordId(request.getVitalRecordId());
		mtlb.setMtlbStatus(MtlbStatus.SUBMITTED);
		if (request.getTimeOfDeath() != null)
			mtlb.setTimeOfDeath(request.getTimeOfDeath().toLocalTime());
		if (request.getType() != null)
			mtlb.setMtlbType(MtlbType.valueOf(request.getType()));
		mtlb.setDeathLocation(request.getDeathLocation());
		mtlb.setDeathRecordId(request.getDeathRecordId());
		return mtlb;
	}

	public ResponseEntity<Long> saveRv4MarraigeMtlbByUnionId(Rv4MarriageMtlbRequest rv4MarriageMtlbRequest) {
		RvfourMarriageMtlb mtlb = rvfourMarriageMtlbRepository.findById(rv4MarriageMtlbRequest.getId());
		if (mtlb != null)
			return ResponseEntity.ok(mtlb.getId());
		else {
			RvfourMarriageMtlb newMtlb = rv4MarriageMtlbRequest2RvfourMarriageMtlb(rv4MarriageMtlbRequest);
			return ResponseEntity.ok(rvfourMarriageMtlbRepository.insert(newMtlb));
		}
	}

	private RvfourMarriageMtlb rv4MarriageMtlbRequest2RvfourMarriageMtlb(
			Rv4MarriageMtlbRequest request) {
		RvfourMarriageMtlb mtlb = new RvfourMarriageMtlb();
		mtlb.setAnnex(request.getAnnex());
		mtlb.setDateOfMarriage(request.getDateOfMarriage());
		mtlb.setHusbandId(request.getHusbandId());
		mtlb.setWifeId(request.getWifeId());
		mtlb.setId(request.getId());
		mtlb.setMarriageLocation(request.getMarriageLocation());
		mtlb.setMarriagePlace(request.getPlace());
		mtlb.setMarriageRites(request.getMarriageRites());
		mtlb.setMtlbStatus(MtlbStatus.SUBMITTED);
		mtlb.setMtlbType(MtlbType.valueOf(request.getType()));
		if (request.getTimeOfMarriage() != null)
			mtlb.setTimeOfMarriage(request.getTimeOfMarriage().toLocalTime());
		mtlb.setWitness1Id(request.getWitness1Id());
		mtlb.setWitness2Id(request.getWitness2Id());
		return mtlb;
	}

	public ResponseEntity<Long> saveRv4DivorceMtlbByUnionId(Rv4DivorceMtlbRequest rv4DivorceMtlbRequest) {
		RvfourDivorceMtlb mtlb = new RvfourDivorceMtlb();
		mtlb.setAnnex(rv4DivorceMtlbRequest.getAnnex());
		mtlb.setDateOfEffect(rv4DivorceMtlbRequest.getDateOfEffect());
		mtlb.setDateOfOrder(rv4DivorceMtlbRequest.getDateOfOrder());
		mtlb.setMtlbType(MtlbType.valueOf(rv4DivorceMtlbRequest.getType()));
		mtlb.setUnionRecordId(rv4DivorceMtlbRequest.getUnionRecordId());
		mtlb.setMtlbStatus(MtlbStatus.SUBMITTED);
		return ResponseEntity.ok(rvfourDivorceMtlbRepository.insert(mtlb));
	}

	public ResponseEntity<Long> saveRv4IdCardMtlbByIdCardNumber(Rv4IdCardMtlbRequest rv4IdCardMtlbRequest) {
		RvfourIdCardMtlb mtlb = rvfourIdCardMtlbRepository.findByIdCardNumber(rv4IdCardMtlbRequest.getIdCardNumber());
		if (mtlb != null)
			return ResponseEntity.ok(mtlb.getId());
		else {
			mtlb = new RvfourIdCardMtlb();
			mtlb.setAnnex(rv4IdCardMtlbRequest.getAnnex());
			mtlb.setCancelledDate(rv4IdCardMtlbRequest.getCancelled_date());
			mtlb.setCancelledReason(rv4IdCardMtlbRequest.getCancelled_reason());
			mtlb.setIssuedDate(rv4IdCardMtlbRequest.getIssuedDate());
			mtlb.setExpires(rv4IdCardMtlbRequest.getExpires());
			mtlb.setIdCardNumber(rv4IdCardMtlbRequest.getIdCardNumber());
			mtlb.setMtlbStatus(MtlbStatus.SUBMITTED);
			mtlb.setVitalRecordId(rv4IdCardMtlbRequest.getVitalRecordId());
			return ResponseEntity.ok(rvfourIdCardMtlbRepository.insert(mtlb));
		}
	}

	public ResponseEntity<String> issueDocument(IssueDocumentInput issueDocumentInput) {
		VitalRecord vr = vitalRecordService.findById(issueDocumentInput.getVitalRecordId());
		IdMtlb idMtlb = idMtlbService.findById(issueDocumentInput.getIdMtlbId());
		IssuedDocuments issuedDocument = new IssuedDocuments();	
		issuedDocument.setVitalRecordId(issueDocumentInput.getVitalRecordId());
		if (issueDocumentInput.getFace() != null)
			issuedDocument.setFace(issueDocumentInput.getFace());
		issuedDocument.setDocumentType(DocumentType.valueOf(issueDocumentInput.getDocumentType()));
		String documentId = RandomStringUtils.randomAlphanumeric(12);
		issuedDocument.setDocumentId(documentId);
		issuedDocument.setIssuedDate(LocalDate.now());
		issuedDocument.setExpires(LocalDate.now().plusYears(EXPIRES_YEARS));
		String fullName="";
		if (!StringUtils.isAllBlank(vr.getFirstname()))
			fullName = vr.getFirstname();
		if (!StringUtils.isAllBlank(vr.getSecondname()))
			fullName = fullName.concat(" " + vr.getSecondname());
		if (!StringUtils.isAllBlank(vr.getThirdname()))
			fullName = fullName.concat(" " + vr.getThirdname());
		if (!StringUtils.isAllBlank(vr.getFourthname()))
			fullName = fullName.concat(" " + vr.getFourthname());
		issuedDocument.setFullName(fullName);
		/*List<IssuedDocuments> oldDocuments = issuedDocumentsRepository.find("vital_record_id='" + issueDocumentInput.getVitalRecordId() + "' ORDER BY id DESC LIMIT 1");
		if (!oldDocuments.isEmpty()) {
			IssuedDocuments oldDocument = oldDocuments.get(0);
			oldDocument.setCancelledDate(LocalDate.now());
			oldDocument.setCancelledReason("Outdated ID");
			issuedDocumentsRepository.update(oldDocument);
		}*/
		List<IssuedDocuments> oldDocuments = issuedDocumentsRepository.findByVitalRecordId(issuedDocument.getVitalRecordId());
		for (IssuedDocuments doc: oldDocuments) {
			if (doc.getCancelledDate() == null) {
				doc.setCancelledDate(LocalDate.now());
				doc.setCancelledReason("Outdated ID");
				issuedDocumentsRepository.update(doc);
			}
		}
		idMtlb.setPrinted(true);
		idMtlbService.update(idMtlb);
		Long id = issuedDocumentsRepository.insert(issuedDocument);
		vr.setCardId(id);
		vitalRecordService.update(vr);
		return ResponseEntity.ok(documentId);
	}

	public ResponseEntity<GetServiceStatusResponse> getServiceStatus(GetServiceStatusRequest request) {
		GetServiceStatusResponse response = new GetServiceStatusResponse();
		switch (request.getType()) {
		case 1 :
			CivilStatusMtlb mtlb = civilStatusMtlbService.findById(request.getId());
			if (mtlb != null)
				response.setServiceStatus(mtlb.getMtlbStatus().name());
			break;
		case 2 :
			IdMtlb idMtlb = idMtlbService.findById(request.getId());
			if (idMtlb != null)
				response.setServiceStatus(idMtlb.getMtlbStatus().name());
			break;
		}
		return ResponseEntity.ok(response);
	}
}