package systems.archide.core.service.hooks;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import systems.archide.core.pojo.AdoptiveFathers;
import systems.archide.core.pojo.AdoptiveMothers;
import systems.archide.core.pojo.CivilStatusMtlb;
import systems.archide.core.pojo.CivilStatusMtlbAttachment;
import systems.archide.core.pojo.CivilStatusMtlbDeathNotificationFile;
import systems.archide.core.pojo.CivilStatusMtlbDocument;
import systems.archide.core.pojo.CivilStatusMtlbMedicalNotificationFile;
import systems.archide.core.pojo.DeathRecord;
import systems.archide.core.pojo.DeathRecordAttachment;
import systems.archide.core.pojo.DeathRecordDeathNotificationFile;
import systems.archide.core.pojo.FileStatus;
import systems.archide.core.pojo.MtlbStatus;
import systems.archide.core.pojo.MtlbType;
import systems.archide.core.pojo.VitalRecordAttachment;
import systems.archide.core.pojo.VitalRecordMedicalNotificationFile;
import systems.archide.core.repository.AdoptiveFathersRepository;
import systems.archide.core.repository.AdoptiveMothersRepository;
import systems.archide.core.repository.CivilStatusMtlbAttachmentRepository;
import systems.archide.core.repository.CivilStatusMtlbDeathNotificationFileRepository;
import systems.archide.core.repository.CivilStatusMtlbDocumentRepository;
import systems.archide.core.repository.CivilStatusMtlbMedicalNotificationFileRepository;
import systems.archide.core.repository.DeathRecordAttachmentRepository;
import systems.archide.core.repository.DeathRecordDeathNotificationFileRepository;
import systems.archide.core.repository.DeathRecordRepository;
import systems.archide.core.repository.VitalRecordAttachmentRepository;
import systems.archide.core.repository.VitalRecordMedicalNotificationFileRepository;

@Component
public class CivilStatusMtlbDocumentLogic {
	
	@Autowired
	CivilStatusMtlbDocumentRepository civilStatusMtlbDocumentRepository;
	
	@Autowired
	CivilStatusMtlbAttachmentRepository civilStatusMtlbAttachmentRepository;
	
	@Autowired
	VitalRecordAttachmentRepository vitalRecordAttachmentRepository;
	
	@Autowired
	VitalRecordMedicalNotificationFileRepository vitalRecordMedicalNotificationFileRepository;
	
	@Autowired
	CivilStatusMtlbMedicalNotificationFileRepository civilStatusMtlbMedicalNotificationFileRepository;
	
	@Autowired
	DeathRecordRepository deathRecordRepository;
	
	@Autowired
	CivilStatusMtlbDeathNotificationFileRepository civilStatusMtlbDeathNotificationFileRepository;
	
	@Autowired
	DeathRecordDeathNotificationFileRepository deathRecordDeathNotificationFileRepository;
	
	@Autowired
	DeathRecordAttachmentRepository deathRecordAttachmentRepository;
	
    @Autowired
    AdoptiveFathersRepository adoptiveFathersRepository;
    
    @Autowired
    AdoptiveMothersRepository adoptiveMothersRepository;

	public Long newDocument(CivilStatusMtlbDocument document) {
		CivilStatusMtlb mtlb = new CivilStatusMtlb();
		dto2pojo(document, mtlb);
		mtlb.setMtlbStatus(MtlbStatus.INITIATED);
		mtlb.setDraft(true);
		mtlb.setMtlbType(document.getMtlbType());
		Long id = civilStatusMtlbDocumentRepository.insert(mtlb);
		if ((MtlbType.CIVIL_STATUS_CHANGE.equals(mtlb.getMtlbType()) || MtlbType.ADOPTION.equals(mtlb.getMtlbType()))  && mtlb.getVitalRecordId() != null) {
			List<VitalRecordAttachment> vrAttchmentList = vitalRecordAttachmentRepository.findByVitalRecordId(mtlb.getVitalRecordId());
			for (VitalRecordAttachment a : vrAttchmentList) {
				CivilStatusMtlbAttachment attachment = new CivilStatusMtlbAttachment();
				attachment.setCivilStatusMtlbId(id);
				attachment.setContent(a.getContent());
				attachment.setDescription(a.getDescription());
				attachment.setFileName(a.getFileName());
				attachment.setFileStatus(FileStatus.DRAFT);
				attachment.setPreview(a.getPreview());
				civilStatusMtlbAttachmentRepository.insert(attachment);
			}
			
			List<VitalRecordMedicalNotificationFile> vrMList = vitalRecordMedicalNotificationFileRepository.findByVitalRecordId(mtlb.getVitalRecordId());
			for (VitalRecordMedicalNotificationFile m : vrMList) {
				CivilStatusMtlbMedicalNotificationFile attachment = new CivilStatusMtlbMedicalNotificationFile();
				attachment.setCivilStatusMtlbId(id);
				attachment.setContent(m.getContent());
				attachment.setDescription(m.getDescription());
				attachment.setFileName(m.getFileName());
				attachment.setFileStatus(FileStatus.DRAFT);
				attachment.setPreview(m.getPreview());
				civilStatusMtlbMedicalNotificationFileRepository.insert(attachment);
			}
		}
		if (MtlbType.DEATH_RECORD_CHANGE.equals(mtlb.getMtlbType()) && mtlb.getVitalRecordId() != null) {
			addDeathRecordAttachmentsToMtlb(mtlb, mtlb.getVitalRecordId());
		}
		return id;
	}

	private void addDeathRecordAttachmentsToMtlb(CivilStatusMtlb mtlb, Long vitalRecordId) {
		List<DeathRecord> deathRecordList = deathRecordRepository.findByVitalRecordId(vitalRecordId);
		if (!deathRecordList.isEmpty()) {
			DeathRecord deathRecord = deathRecordList.get(0);
			List<DeathRecordAttachment> drAttchmentList = deathRecordAttachmentRepository.findByDeathRecordId(deathRecord.getId());
			for (DeathRecordAttachment a : drAttchmentList) {
				CivilStatusMtlbAttachment attachment = new CivilStatusMtlbAttachment();
				attachment.setCivilStatusMtlbId(mtlb.getId());
				attachment.setContent(a.getContent());
				attachment.setDescription(a.getDescription());
				attachment.setFileName(a.getFileName());
				attachment.setFileStatus(FileStatus.DRAFT);
				attachment.setPreview(a.getPreview());
				civilStatusMtlbAttachmentRepository.insert(attachment);
			}
			
			List<DeathRecordDeathNotificationFile> drDList = deathRecordDeathNotificationFileRepository.findByDeathRecordId(deathRecord.getId());
			if (!drDList.isEmpty())
				mtlb.setDeathNotification(true);
			for (DeathRecordDeathNotificationFile m : drDList) {
				CivilStatusMtlbDeathNotificationFile attachment = new CivilStatusMtlbDeathNotificationFile();
				attachment.setCivilStatusMtlbId(mtlb.getId());
				attachment.setContent(m.getContent());
				attachment.setDescription(m.getDescription());
				attachment.setFileName(m.getFileName());
				attachment.setFileStatus(FileStatus.DRAFT);
				attachment.setPreview(m.getPreview());
				civilStatusMtlbDeathNotificationFileRepository.insert(attachment);
			}

		}
	}

	public Long editDocument(CivilStatusMtlbDocument document) {
		CivilStatusMtlb mtlb = civilStatusMtlbDocumentRepository.findById(document.getId());
		mtlb.setMtlbStatus(MtlbStatus.SUBMITTED);
		dto2pojo(document, mtlb);
		civilStatusMtlbDocumentRepository.update(mtlb);
		return mtlb.getId();
	}

	private void dto2pojo(CivilStatusMtlbDocument document, CivilStatusMtlb mtlb) {
		mtlb.setUnknownFather(document.isUnknownFather());
		mtlb.setUnknownMother(document.isUnknownMother());
        mtlb.setBirthDayUnknown(document.isBirthDayUnknown());
        mtlb.setBirthCountry(document.getBirthCountry());
        mtlb.setBirthMonthUnknown(document.isBirthMonthUnknown());
        mtlb.setBirthPlace(document.getBirthPlace());
        mtlb.setBirthTime(document.getBirthTime());
        mtlb.setBirthTimeUnknown(document.isBirthTimeUnknown());
        mtlb.setBirthYearUnknown(document.isBirthYearUnknown());
        mtlb.setBirthdate(document.getBirthdate());
        mtlb.setDateOfDeath(document.getDateOfDeath());
        mtlb.setDeathDayUnknown(document.isDeathDayUnknown());
        mtlb.setDeathMonthUnknown(document.isDeathMonthUnknown());
        mtlb.setDeathTimeUnknown(document.isDeathTimeUnknown());
        mtlb.setBook(document.getBook());
        mtlb.setDeathYearUnknown(document.isDeathYearUnknown());
        mtlb.setDeclarantId(document.getDeclarantId());
        mtlb.setDeduplicationMtlbId(document.getDeduplicationMtlbId());
        mtlb.setDisability(document.isDisability());
        mtlb.setDonor(document.isDonor());
        mtlb.setDonorNotes(document.getDonorNotes());
        mtlb.setDraft(document.isDraft());
        mtlb.setFace(document.getFace());
        mtlb.setFaceMimeType(document.getFaceMimeType());
        mtlb.setFatherId(document.getFatherId());
        mtlb.setFifthname(document.getFifthname());
        mtlb.setFingersMimeType(document.getFingersMimeType());
        mtlb.setFirstname(document.getFirstname());
        mtlb.setFourthname(document.getFourthname());
        mtlb.setGender(document.getGender());
        mtlb.setLeftIndexFinger(document.getLeftIndexFinger());
        mtlb.setLeftMiddleFinger(document.getLeftMiddleFinger());
        mtlb.setLeftPinky(document.getLeftPinky());
        mtlb.setLeftRingFinger(document.getLeftRingFinger());
        mtlb.setLeftThumb(document.getLeftThumb());
        mtlb.setMaritalStatus(document.getMaritalStatus());
        mtlb.setMedicalNotification(document.isMedicalNotification());
        mtlb.setMotherId(document.getMotherId());
        mtlb.setMtlbStatusData(document.getMtlbStatusData());
        mtlb.setMtlbType(document.getMtlbType());
        mtlb.setNotes(document.getNotes());
        mtlb.setNumberOfAttachments(document.getNumberOfAttachments());
        mtlb.setOtherCitizenshipCsv(document.getOtherCitizenshipCsv());
        mtlb.setPage(document.getPage());
        mtlb.setPrimaryCitizenship(document.getPrimaryCitizenship());
        mtlb.setReportedFatherName(document.getReportedFatherName());
        mtlb.setReportedId(document.getReportedId());
        mtlb.setReportedLegacyId(document.getReportedLegacyId());
        mtlb.setReportedMotherName(document.getReportedMotherName());
        mtlb.setRightIndexFinger(document.getRightIndexFinger());
        mtlb.setRightMiddleFinger(document.getRightMiddleFinger());
        mtlb.setRightPinky(document.getRightPinky());
        mtlb.setRightRingFinger(document.getRightRingFinger());
        mtlb.setRightThumb(document.getRightThumb());
        mtlb.setSecondname(document.getSecondname());
        mtlb.setSignature(document.getSignature());
        mtlb.setSignatureMimeType(document.getSignatureMimeType());
        mtlb.setSixthname(document.getSixthname());
        mtlb.setThirdname(document.getThirdname());
        mtlb.setTimeOfDeath(document.getTimeOfDeath());
        mtlb.setTouched(document.getTouched());
        mtlb.setUploaded(document.isUploaded());
        mtlb.setUuid(document.getUuid());
        mtlb.setVitalRecordId(document.getVitalRecordId());
        mtlb.setWitness1Id(document.getWitness1Id());
        mtlb.setWitness2Id(document.getWitness2Id());
        mtlb.setYear(document.getYear());
        mtlb.setAfisId(document.getAfisId());
        mtlb.setTimeOfDeath(document.getTimeOfDeath());
        mtlb.setDeathPlace(document.getDeathPlace());
        mtlb.setTypeOfAcquisition(document.getTypeOfAcquisition());
        mtlb.setVillageOfOrigin(document.getVillageOfOrigin());
        mtlb.setBurialPlace(document.getBurialPlace());
        mtlb.setDeathNotification(document.isDeathNotification());
        mtlb.setAdoptiveFatherId(document.getAdoptiveFatherId());
        mtlb.setAdoptiveMotherId(document.getAdoptiveMotherId());
        mtlb.setVillageOfOriginAdoptive(document.getVillageOfOriginAdoptive());
        mtlb.setMarriageCertificateNumberAdoptive(document.getMarriageCertificateNumberAdoptive());
        mtlb.setCourtAdoptionNumber(document.getCourtAdoptionNumber());
        mtlb.setRegistrationLocation(document.getRegistrationLocation());
        mtlb.setMentions(document.getMentions());
        if (MtlbType.ADOPTION.equals(document.getMtlbType())) {
        	mtlb.setFatherEndAdoptionDate(document.getFatherEndAdoptionDate());
            mtlb.setMotherEndAdoptionDate(document.getMotherEndAdoptionDate());
        	if (document.getAdoptiveFatherId() != null) {
        		AdoptiveFathers newAdoptiveFather = new AdoptiveFathers();
        		newAdoptiveFather.setChildId(document.getVitalRecordId());
        		newAdoptiveFather.setCivilStatusMtlbId(document.getId());
        		newAdoptiveFather.setCourtAdoptionNumber(document.getCourtAdoptionNumber());
        		newAdoptiveFather.setFatherAdoptionDate(document.getFatherAdoptionDate());
        		newAdoptiveFather.setVitalRecordId(document.getAdoptiveFatherId());
        		adoptiveFathersRepository.insert(newAdoptiveFather);
        	}
        	if (document.getAdoptiveMotherId() != null) {
        		AdoptiveMothers newAdoptiveMother = new AdoptiveMothers();
        		newAdoptiveMother.setChildId(document.getVitalRecordId());
        		newAdoptiveMother.setCivilStatusMtlbId(document.getId());
        		newAdoptiveMother.setCourtAdoptionNumber(document.getCourtAdoptionNumber());
        		newAdoptiveMother.setMotherAdoptionDate(document.getMotherAdoptionDate());
        		newAdoptiveMother.setVitalRecordId(document.getAdoptiveMotherId());
        		adoptiveMothersRepository.insert(newAdoptiveMother);
        	}
        }
	}

}
