package systems.archide.core.service.hooks;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import systems.archide.core.pojo.MtlbStatus;
import systems.archide.core.pojo.MtlbType;
import systems.archide.core.pojo.UnionMtlb;
import systems.archide.core.pojo.UnionMtlbAttachment;
import systems.archide.core.pojo.UnionMtlbDocument;
import systems.archide.core.pojo.UnionRecordAttachment;
import systems.archide.core.pojo.VitalRecord;
import systems.archide.core.repository.UnionMtlbAttachmentRepository;
import systems.archide.core.repository.UnionMtlbDocumentRepository;
import systems.archide.core.repository.UnionMtlbRepository;
import systems.archide.core.repository.UnionRecordAttachmentRepository;
import systems.archide.core.repository.VitalRecordRepository;

@Component
public class UnionMtlbDocumentLogic {
	
	@Autowired
	UnionMtlbDocumentRepository service;
	
	@Autowired
	private VitalRecordRepository vitalRecordService;
	
	@Autowired
	UnionRecordAttachmentRepository unionRecordAttachmentRepository;
	
	@Autowired
	UnionMtlbAttachmentRepository unionMtlbAttachmentRepository;

	public Long newDocument(UnionMtlbDocument document) {
		UnionMtlb uMtlb = new UnionMtlb();
		if (MtlbType.UNION_RECORD_CHANGE.equals(document.getMtlbType()) && document.getUnionRecordId() != null)
			dto2pojo(uMtlb, document);
		uMtlb.setMtlbStatus(MtlbStatus.INITIATED);
		uMtlb.setUnionRecordId(document.getUnionRecordId());
		uMtlb.setMtlbType(document.getMtlbType());
		uMtlb.setDraft(true);
		Long id = service.insert(uMtlb);
		if (MtlbType.UNION_RECORD_CHANGE.equals(document.getMtlbType()) && document.getUnionRecordId() != null)
			addAttachments(id, document.getUnionRecordId());
		return id;
	}

	private void addAttachments(Long mtlbId, Long unionRecordId) {
		List<UnionRecordAttachment> uaList = unionRecordAttachmentRepository.findByUnionRecordId(unionRecordId);
		for (UnionRecordAttachment ua: uaList) {
			UnionMtlbAttachment a = new UnionMtlbAttachment();
			a.setContent(ua.getContent());
			a.setDescription(ua.getDescription());
			a.setFileName(ua.getFileName());
			a.setPreview(ua.getPreview());
			a.setUnionMtlbId(mtlbId);
			unionMtlbAttachmentRepository.insert(a);
		}
	}

	public Long editDocument(UnionMtlbDocument document) {
		UnionMtlb uMtlb = service.findById(document.getId());
		dto2pojo(uMtlb, document);
		uMtlb.setDraft(false);
		uMtlb.setMtlbStatus(MtlbStatus.SUBMITTED);
        uMtlb.setId(document.getId());
		service.update(uMtlb);
		return uMtlb.getId();
	}

	private void dto2pojo(UnionMtlb uMtlb, UnionMtlbDocument document) {
		uMtlb.setAddress(document.getAddress());
        uMtlb.setDeclaredDate(document.getDeclaredDate());
        uMtlb.setMtlbStatusData(document.getMtlbStatusData());
        uMtlb.setMtlbType(document.getMtlbType());
        uMtlb.setNotes(document.getNotes());        
        uMtlb.setPartner1BirthPlace(document.getPartner1BirthPlace());
        uMtlb.setPartner1FatherId(document.getPartner1FatherId());
        uMtlb.setPartner1FatherName(document.getPartner1FatherName());
        uMtlb.setPartner1Firstname(document.getPartner1Firstname());
        uMtlb.setPartner1IndividualMelanesianName(document.getPartner1IndividualMelanesianName());
        uMtlb.setPartner1MaritalStatus(document.getPartner1MaritalStatus());
        uMtlb.setPartner1MotherId(document.getPartner1MotherId());
        uMtlb.setPartner1MotherName(document.getPartner1MotherName());
        uMtlb.setPartner1Occupation(document.getPartner1Occupation());
        uMtlb.setPartner1ResidenceAddress(document.getPartner1ResidenceAddress());
        uMtlb.setPartner1Surname(document.getPartner1Surname());
        uMtlb.setPartner1VitalRecordId(document.getPartner1VitalRecordId());
        uMtlb.setPartner2BirthPlace(document.getPartner2BirthPlace());
        uMtlb.setPartner2FatherId(document.getPartner2FatherId());
        uMtlb.setPartner2FatherName(document.getPartner2FatherName());
        uMtlb.setPartner2Firstname(document.getPartner2Firstname());
        uMtlb.setPartner2IndividualMelanesianName(document.getPartner2IndividualMelanesianName());
        uMtlb.setPartner2MaritalStatus(document.getPartner2MaritalStatus());
        uMtlb.setPartner2MotherId(document.getPartner2MotherId());
        uMtlb.setPartner2MotherName(document.getPartner2MotherName());
        uMtlb.setPartner2Occupation(document.getPartner2Occupation());
        uMtlb.setPartner2ResidenceAddress(document.getPartner2ResidenceAddress());
        uMtlb.setPartner2Surname(document.getPartner2Surname());
        uMtlb.setPartner2VitalRecordId(document.getPartner2VitalRecordId());
        uMtlb.setPlace(document.getPlace());
        uMtlb.setRites(document.getRites());
        uMtlb.setUnionRegime(document.getUnionRegime());
        uMtlb.setWitness1BirthPlace(document.getWitness1BirthPlace());
        uMtlb.setWitness1Firstname(document.getWitness1Firstname());
        uMtlb.setWitness1Occupation(document.getWitness1Occupation());
        uMtlb.setWitness1ResidenceAddress(document.getWitness1ResidenceAddress());
        uMtlb.setWitness1Surname(document.getWitness1Surname());
        uMtlb.setWitness1VitalRecordId(document.getWitness1VitalRecordId());
        uMtlb.setWitness2BirthPlace(document.getWitness2BirthPlace());
        uMtlb.setWitness2Firstname(document.getWitness2Firstname());
        uMtlb.setWitness2Occupation(document.getWitness2Occupation());
        uMtlb.setWitness2ResidenceAddress(document.getWitness2ResidenceAddress());
        uMtlb.setWitness2Surname(document.getWitness2Surname());
        uMtlb.setWitness2VitalRecordId(document.getWitness2VitalRecordId());
        uMtlb.setTimeOfMarriage(document.getTimeOfMarriage());
        uMtlb.setDateOfOrder(document.getDateOfOrder());
        uMtlb.setDateOfEffect(document.getDateOfEffect());
        uMtlb.setAnnullmentOrder(document.getAnnullmentOrder());
        uMtlb.setUnionRecordId(document.getUnionRecordId());
        uMtlb.setCelebrant(document.getCelebrant());
        uMtlb.setPosition(document.getPosition());
        uMtlb.setOfficer(document.getOfficer());
        uMtlb.setCelebrantId(document.getCelebrantId());
        uMtlb.setDeclarantId(document.getDeclarantId());
        uMtlb.setOfficerId(document.getOfficerId());
        uMtlb.setRegistrationLocation(document.getRegistrationLocation());
        uMtlb.setMentions(document.getMentions());
        if (document.getPartner1VitalRecordId() != null) {
			VitalRecord partner1 = vitalRecordService.findById(document.getPartner1VitalRecordId());
			if (partner1 != null) {
				uMtlb.setPartner1Firstname(partner1.getFirstname());
				uMtlb.setPartner1Surname(partner1.getFourthname());	
			}
		}
		if (document.getPartner2VitalRecordId() != null) {
			VitalRecord partner2 = vitalRecordService.findById(document.getPartner2VitalRecordId());
			if (partner2 != null)  {
				uMtlb.setPartner2Firstname(partner2.getFirstname());
				uMtlb.setPartner2Surname(partner2.getFourthname());
			}
		}
	}

}
