package systems.archide.core.service.hooks;

import java.time.LocalDate;
import java.time.Period;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import org.bouncycastle.asn1.ocsp.ResponderID;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.http.ResponseEntity.BodyBuilder;
import org.springframework.stereotype.Component;

import systems.archide.core.controller.search.VitalRecordOrderBy;
import systems.archide.core.pojo.AddAddressRequest;
import systems.archide.core.pojo.Address;
import systems.archide.core.pojo.CountVoterInBatchRequest;
import systems.archide.core.pojo.DeathRecord;
import systems.archide.core.pojo.DocumentType;
import systems.archide.core.pojo.Face;
import systems.archide.core.pojo.GetVoterListRequest;
import systems.archide.core.pojo.ImportVoterRequest;
import systems.archide.core.pojo.ImportVoterResponse;
import systems.archide.core.pojo.IssuedDocuments;
import systems.archide.core.pojo.VitalRecord;
import systems.archide.core.pojo.VitalRecordHistory;
import systems.archide.core.pojo.VoterRecord;
import systems.archide.core.repository.AddressRepository;
import systems.archide.core.repository.DeathRecordRepository;
import systems.archide.core.repository.FaceRepository;
import systems.archide.core.repository.IdcoreCustomQueries;
import systems.archide.core.repository.IssuedDocumentsRepository;
import systems.archide.core.repository.VitalRecordHistoryRepository;
import systems.archide.core.repository.VitalRecordRepository;

@Component
public class VanElectServiceLogic {

	@Autowired
	VitalRecordRepository vitalRecordRepository;

	@Autowired
	DeathRecordRepository deathRecordRepository;

	@Autowired
	IssuedDocumentsRepository issuedDocumentsRepository;

	@Autowired
	FaceRepository faceRepository;

	@Autowired
	AddressRepository addressRepository;

	@Autowired
	IdcoreCustomQueries queries;

	@Autowired
	VitalRecordHistoryRepository vitalRecordHistoryRepository;

	public ResponseEntity<List<VoterRecord>> getVoterRecordList(GetVoterListRequest getVoterListRequest) {
		List<VoterRecord> voterRecordList = new ArrayList<>();
		HttpHeaders metadata = new HttpHeaders();
		ResponseEntity<List<VoterRecord>> response = null;
		long maxId = queries.getMaxVitalRecordId();
		List<VitalRecord> vitalRecordList = queries.getVoterList(getVoterListRequest.getCutOffDate(),
				getVoterListRequest.getCutOffAgeDate().toLocalDate(), getVoterListRequest.getMin(),
				getVoterListRequest.getMax());
		List<VitalRecord> vitalRecordHistoryList = queries.getVoterListFromHistory(getVoterListRequest.getCutOffDate(),
				getVoterListRequest.getCutOffAgeDate().toLocalDate(), getVoterListRequest.getMin(),
				getVoterListRequest.getMax(), getVoterListRequest.getCutOffDate());
		for (VitalRecord vr : vitalRecordList) {
			VoterRecord voter = getValideVoter(vr, getVoterListRequest);
			if (voter != null)
				voterRecordList.add(voter);
		}
		for (VitalRecord vr : vitalRecordHistoryList) {
			VoterRecord voter = getValideVoter(vr, getVoterListRequest);
			if (voter != null)
				voterRecordList.add(voter);
		}
		if (maxId < getVoterListRequest.getMax())
			metadata.add("status", "DONE");
		else
			metadata.add("status", "CONTINUE");
		response = ResponseEntity.status(HttpStatus.CREATED).headers(metadata).body(voterRecordList);
		return response;
	}

	private VoterRecord getValideVoter(VitalRecord vr, GetVoterListRequest getVoterListRequest) {
		List<DeathRecord> drList = deathRecordRepository.findByVitalRecordId(vr.getId());
		if (!drList.isEmpty())
			return null;
		List<IssuedDocuments> issuedDocumentList = issuedDocumentsRepository.findByVitalRecordId(vr.getId());
		for (IssuedDocuments issuedDocument : issuedDocumentList) {
			if (DocumentType.ID_CARD.equals(issuedDocument.getDocumentType()) && "191".equals(vr.getPrimaryCitizenship())) {
				VoterRecord voter = new VoterRecord();
				voter.setFirstname(vr.getFirstname());
				voter.setSecondname(vr.getSecondname());
				voter.setThirdname(vr.getThirdname());
				voter.setFourthname(vr.getFourthname());
				voter.setIdCard(issuedDocument.getDocumentId());
				voter.setId(vr.getId());
				voter.setBirthdate(vr.getBirthdate());
				if (vr.getFaceId() != null) {
					Face face = faceRepository.findById(vr.getFaceId());
					voter.setFace(face.getImage());
				}
				List<Address> vrAdresses = addressRepository.findByVitalRecordId(vr.getId());
				String lastAddress = null;
				if (!vrAdresses.isEmpty())
					lastAddress = vrAdresses.get(vrAdresses.size() - 1).getAreaId();
				voter.setAddress(lastAddress);
				if (getVoterListRequest.getGeography() != null) {
					if (lastAddress != null)
						if (lastAddress.startsWith(getVoterListRequest.getGeography()))
							return voter;
				}
				else
					return voter;
				return null;
			}
		}
		return null;
	}

	public ResponseEntity<Long> getMaxId() {
		return ResponseEntity.ok(queries.getMaxVitalRecordId());
	}

	public ResponseEntity<Long> countVotersInBatch(CountVoterInBatchRequest request) {
		HashSet<Long> votersInBatch = new HashSet<>();
		List<Long> mainNumberOfVoters = queries.getMainIdListInBatch(request.getCutOffDate(), request.getCutOffAgeDate().toLocalDate(),
				request.getMin(), request.getMax());
		List<Long> historyNumberOfVoters = queries.getHistoryIdListInBatch(request.getCutOffDate(), request.getCutOffAgeDate().toLocalDate(),
				request.getMin(), request.getMax(), request.getCutOffDate());
		for (Long id: mainNumberOfVoters)
			votersInBatch.add(id);
		for (Long hId: historyNumberOfVoters)
			votersInBatch.add(hId);
		return ResponseEntity.ok((long) votersInBatch.size());
	}

	public ResponseEntity<Long> addAddress(AddAddressRequest addAddressRequest) {
		VitalRecord vr = vitalRecordRepository.findById(addAddressRequest.getVitalRecordId());
		if (vr != null) {
			Address address = new Address();
			address.setAreaId(addAddressRequest.getAddress());
			address.setVitalRecordId(vr.getId());
			addressRepository.insert(address);
			return ResponseEntity.ok(vr.getId());
		}
		return ResponseEntity.notFound().build();
	}

	public ResponseEntity<ImportVoterResponse> importVoter(ImportVoterRequest importVoterRequest) {
		VitalRecord vr = vitalRecordRepository.findById(importVoterRequest.getVitalRecordId());
		if (vr != null) {
			ImportVoterResponse v = new ImportVoterResponse();
			v.setBirthdate(vr.getBirthdate());
			v.setFirstname(vr.getFirstname());
			v.setFourthname(vr.getFourthname());
			v.setSecondname(vr.getSecondname());
			v.setThirdname(vr.getThirdname());
			v.setId(vr.getId());
			if (vr.getFaceId() != null) {
				Face face = faceRepository.findById(vr.getFaceId());
				v.setFace(face.getImage());
			}
			List<Address> vrAdresses = addressRepository.findByVitalRecordId(vr.getId());
			List<IssuedDocuments> issuedDocumentList = issuedDocumentsRepository.findByVitalRecordId(vr.getId());
			for (IssuedDocuments issuedDocument : issuedDocumentList) {
				if (DocumentType.ID_CARD.equals(issuedDocument.getDocumentType()))
					v.setIdCard(issuedDocument.getDocumentId());
			}
			String lastAddress = null;
			if (!vrAdresses.isEmpty())
				lastAddress = vrAdresses.get(vrAdresses.size() - 1).getAreaId();
			v.setAddress(lastAddress);
			return ResponseEntity.ok(v);
		}
		return ResponseEntity.notFound().build();
	}
}
