package systems.archide.core.workflows;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import systems.archide.core.pojo.DocumentType;
import systems.archide.core.pojo.GeneralStatistics;
import systems.archide.core.pojo.GeoData;
import systems.archide.core.pojo.IssuedDocumentsDto;
import systems.archide.core.pojo.JobMtlb;
import systems.archide.core.pojo.MtlbStatus;
import systems.archide.core.pojo.ProvinceGeneralStatistics;
import systems.archide.core.pojo.VitalRecordDto;
import systems.archide.core.repository.GeoDataRepository;
import systems.archide.core.repository.IdcoreCustomQueries;
import systems.archide.core.repository.IssuedDocumentsRepository;
import systems.archide.core.repository.JobMtlbRepository;
import systems.archide.core.repository.VitalRecordRepository;

@Component
public class JobMtlbSubmitted {
	
	private static final Logger log = LoggerFactory.getLogger(JobMtlbSubmitted.class);
	
	private static final Short MALE = 1;
	private static final Short FEMALE = 2;

	@Autowired
	JobMtlbRepository jobMtlbRepository;
	
	@Autowired
	IdcoreCustomQueries queries;
	
	@Autowired
	GeoDataRepository geoDataRepository;
	
	@Autowired
	IssuedDocumentsRepository issuedDocumentsRepository;
	
	@Autowired
	VitalRecordRepository vitalRecordRepository;
	
	@Scheduled(fixedDelay = 5000)
	public void fromSubmittedMtlb() {
		List<JobMtlb> mtlbList = jobMtlbRepository.findByMtlbStatus(MtlbStatus.SUBMITTED);
		for (JobMtlb mtlb : mtlbList) {
			switch (mtlb.getMtlbType()) {
				case GENERAL_STATISTICS: 
					mtlb.setMtlbStatus(MtlbStatus.PROCESSING);
					jobMtlbRepository.update(mtlb);
					processGeneralStatisticsMtlb(mtlb);
				break;
			default:
				break;
			}
		}
	}

	private void processGeneralStatisticsMtlb(JobMtlb mtlb) {
		GeneralStatistics generalStatistics = new GeneralStatistics();
		IssuedDocumentsDto filter = new IssuedDocumentsDto();
		filter.addDocumentType(DocumentType.ID_CARD);
		Long issuedIdCardsNumber = issuedDocumentsRepository.countSearchByDocumentType(filter, null, null, null, null, true);
		
		Long notExpiredIdCardsNumber = queries.getNotExpiredIdCardNumber();
		Long notCancelledIdCardsNumber = queries.getNotCancelledIdCardNumber();
		Long activatedIdCardsNumber = queries.getActiveIdCardNumber();
		Long distinctIndividualsIdCardsNumber = queries.getDistinctVitalRecordIdCardsNumber();
		VitalRecordDto recordFilter = new VitalRecordDto();
		recordFilter.addInactive(false);
		Long authorizedBirthRecordsNumber = vitalRecordRepository.count();
		Long livingAuthorizedBirthRecordsNumber = queries.getLivingVitalRecordNumber();
		Long livingFemalesAged18To29Number = queries.getLivingVitalRecordByGenderAndAgeRange(FEMALE, 18, 29);
		Long livingMalesAged18To29Number = queries.getLivingVitalRecordByGenderAndAgeRange(MALE, 18, 29);
		Long livingMalesAgedUnder18Number = queries.getLivingVitalRecordByGenderAndAgeRange(MALE, 0, 17);
		Long livingFemalesAgedUnder18Number = queries.getLivingVitalRecordByGenderAndAgeRange(FEMALE, 0, 17);
		Long livingMalesAged30To49Number = queries.getLivingVitalRecordByGenderAndAgeRange(MALE, 30, 49);
		Long livingFemalesAged30To49Number = queries.getLivingVitalRecordByGenderAndAgeRange(FEMALE, 30, 49);
		Long livingMalesAgedOver49Number = queries.getLivingVitalRecordByGenderAndAgeRange(MALE, 50, 120);
		Long livingFemalesAgedOver49Number = queries.getLivingVitalRecordByGenderAndAgeRange(FEMALE, 50, 120);
		Long livingPeopleWithCardsNumber = queries.getLivingVitalRecordIdCardsNumber();
		Long livingFemalesAgedUnder18WithCardsNumber = queries
				.getLivingVitalRecordWithIdCardByGenderAndAgeRange(FEMALE, 0, 17);
		Long livingMalesAgedUnder18WithCardsNumber = queries
				.getLivingVitalRecordWithIdCardByGenderAndAgeRange(MALE, 0, 17);
		Long livingFemalesAged18To29WithCardsNumber = queries
				.getLivingVitalRecordWithIdCardByGenderAndAgeRange(FEMALE, 18, 29);
		Long livingMalesAged18To29WithCardsNumber = queries
				.getLivingVitalRecordWithIdCardByGenderAndAgeRange(MALE, 18, 29);
		Long livingFemalesAged30To49WithCardsNumber = queries
				.getLivingVitalRecordWithIdCardByGenderAndAgeRange(FEMALE, 30, 49);
		Long livingMalesAged30To49WithCardsNumber = queries
				.getLivingVitalRecordWithIdCardByGenderAndAgeRange(MALE, 30, 49);
		Long livingFemalesAgedOver49WithCardsNumber = queries
				.getLivingVitalRecordWithIdCardByGenderAndAgeRange(FEMALE, 50, 120);
		Long livingMalesAgedOver49WithCardsNumber = queries
				.getLivingVitalRecordWithIdCardByGenderAndAgeRange(MALE, 50, 120);
		generalStatistics.setIssuedIdCardsNumber(issuedIdCardsNumber);
		generalStatistics.setNotExpiredIdCardsNumber(notExpiredIdCardsNumber);
		generalStatistics.setNotCancelledIdCardsNumber(notCancelledIdCardsNumber);
		generalStatistics.setActivatedIdCardsNumber(activatedIdCardsNumber);
		generalStatistics.setDistinctIndividualsIdCardsNumber(distinctIndividualsIdCardsNumber);
		generalStatistics.setAuthorizedBirthRecordsNumber(authorizedBirthRecordsNumber);
		generalStatistics.setLivingFemalesAged18To29Number(livingFemalesAged18To29Number);
		generalStatistics.setLivingMalesAged18To29Number(livingMalesAged18To29Number);
		generalStatistics.setLivingMalesAgedUnder18Number(livingMalesAgedUnder18Number);
		generalStatistics.setLivingFemalesAgedUnder18Number(livingFemalesAgedUnder18Number);
		generalStatistics.setLivingMalesAged30To49Number(livingMalesAged30To49Number);
		generalStatistics.setLivingFemalesAged30To49Number(livingFemalesAged30To49Number);
		generalStatistics.setLivingMalesAgedOver49Number(livingMalesAgedOver49Number);
		generalStatistics.setLivingFemalesAgedOver49Number(livingFemalesAgedOver49Number);
		generalStatistics.setLivingAuthorizedBirthRecordsNumber(livingAuthorizedBirthRecordsNumber);
		generalStatistics.setLivingPeopleWithCardsNumber(livingPeopleWithCardsNumber);
		generalStatistics.setLivingMalesAgedUnder18WithCardsNumber(livingMalesAgedUnder18WithCardsNumber);
		generalStatistics.setLivingFemalesAgedUnder18WithCardsNumber(livingFemalesAgedUnder18WithCardsNumber);
		generalStatistics.setLivingFemalesAged18To29WithCardsNumber(livingFemalesAged18To29WithCardsNumber);
		generalStatistics.setLivingMalesAged18To29WithCardsNumber(livingMalesAged18To29WithCardsNumber);
		generalStatistics.setLivingFemalesAged30To49WithCardsNumber(livingFemalesAged30To49WithCardsNumber);
		generalStatistics.setLivingMalesAged30To49WithCardsNumber(livingMalesAged30To49WithCardsNumber);
		generalStatistics.setLivingFemalesAgedOver49WithCardsNumber(livingFemalesAgedOver49WithCardsNumber);
		generalStatistics.setLivingMalesAgedOver49WithCardsNumber(livingMalesAgedOver49WithCardsNumber);
		List<ProvinceGeneralStatistics> provinceGeneralStatisticsList = getProvinceGeneralStatisticsList();
		generalStatistics.setProvinceGeneralStatisticsList(provinceGeneralStatisticsList);
		
		ObjectMapper objectMapper = new ObjectMapper();
		 String jobContent = null;
		try {
			jobContent = objectMapper.writeValueAsString(generalStatistics);
		} catch (JsonProcessingException e) {
			e.printStackTrace();
		}
		if (jobContent != null) {
			mtlb.setContent(jobContent);
			mtlb.setMtlbStatus(MtlbStatus.ARCHIVED);
			jobMtlbRepository.update(mtlb);
		}
	}
	
	private List<ProvinceGeneralStatistics> getProvinceGeneralStatisticsList() {
		List<ProvinceGeneralStatistics> provinceGeneralStatisticsList = new ArrayList<>();
		List<GeoData> provinceList = geoDataRepository.findByType(2);
		for (GeoData province : provinceList) {
			ProvinceGeneralStatistics provinceGeneralStatistics = new ProvinceGeneralStatistics();
			provinceGeneralStatistics.setProvinceName(province.getName());

			Long authorizedBirthRecordsNumber = queries
					.getVitalRecordByAddressNumber(province.getAreaId() + "%");
			Long livingAuthorizedBirthRecordsNumber = queries
					.getLivingVitalRecordByAddressNumber(province.getAreaId() + "%");
			Long livingFemalesAged18To29Number = queries.getLivingVitalRecordByGenderAndAgeRangeAndAddress(province.getAreaId() + "%", FEMALE, 18, 29);
			Long livingMalesAged18To29Number = queries.getLivingVitalRecordByGenderAndAgeRangeAndAddress(province.getAreaId() + "%",MALE, 18, 29);
			Long livingMalesAgedUnder18Number = queries.getLivingVitalRecordByGenderAndAgeRangeAndAddress(province.getAreaId() + "%",MALE, 0, 17);
			Long livingFemalesAgedUnder18Number = queries.getLivingVitalRecordByGenderAndAgeRangeAndAddress(province.getAreaId() + "%",FEMALE, 0, 17);
			Long livingMalesAged30To49Number = queries.getLivingVitalRecordByGenderAndAgeRangeAndAddress(province.getAreaId() + "%",MALE, 30, 49);
			Long livingFemalesAged30To49Number = queries.getLivingVitalRecordByGenderAndAgeRangeAndAddress(province.getAreaId() + "%",FEMALE, 30, 49);
			Long livingMalesAgedOver49Number = queries.getLivingVitalRecordByGenderAndAgeRangeAndAddress(province.getAreaId() + "%",MALE, 50, 120);
			Long livingFemalesAgedOver49Number = queries.getLivingVitalRecordByGenderAndAgeRangeAndAddress(province.getAreaId() + "%",FEMALE, 50, 120);
			Long livingPeopleWithCardsNumber = queries
					.getLivingVitalRecordIdCardsByAddressNumber(province.getAreaId() + "%");
			Long livingFemalesAgedUnder18WithCardsNumber = queries
					.getLivingVitalRecordWithIdCardByGenderAndAgeRangeAndAddress(province.getAreaId() + "%", FEMALE, 0, 17);
			Long livingMalesAgedUnder18WithCardsNumber = queries
					.getLivingVitalRecordWithIdCardByGenderAndAgeRangeAndAddress(province.getAreaId() + "%", MALE, 0, 17);
			Long livingFemalesAged18To29WithCardsNumber = queries
					.getLivingVitalRecordWithIdCardByGenderAndAgeRangeAndAddress(province.getAreaId() + "%", FEMALE, 18, 29);
			Long livingMalesAged18To29WithCardsNumber = queries
					.getLivingVitalRecordWithIdCardByGenderAndAgeRangeAndAddress(province.getAreaId() + "%", MALE, 18, 29);
			Long livingFemalesAged30To49WithCardsNumber = queries
					.getLivingVitalRecordWithIdCardByGenderAndAgeRangeAndAddress(province.getAreaId() + "%", FEMALE, 30, 49);
			Long livingMalesAged30To49WithCardsNumber = queries
					.getLivingVitalRecordWithIdCardByGenderAndAgeRangeAndAddress(province.getAreaId() + "%", MALE, 30, 49);
			Long livingFemalesAgedOver49WithCardsNumber = queries
					.getLivingVitalRecordWithIdCardByGenderAndAgeRangeAndAddress(province.getAreaId() + "%", FEMALE, 50, 120);
			Long livingMalesAgedOver49WithCardsNumber =  queries
					.getLivingVitalRecordWithIdCardByGenderAndAgeRangeAndAddress(province.getAreaId() + "%", MALE, 50, 120);
			provinceGeneralStatistics.setAuthorizedBirthRecordsNumber(authorizedBirthRecordsNumber);
			provinceGeneralStatistics.setLivingFemalesAged18To29Number(livingFemalesAged18To29Number);
			provinceGeneralStatistics.setLivingMalesAged18To29Number(livingMalesAged18To29Number);
			provinceGeneralStatistics.setLivingMalesAgedUnder18Number(livingMalesAgedUnder18Number);
			provinceGeneralStatistics.setLivingFemalesAgedUnder18Number(livingFemalesAgedUnder18Number);
			provinceGeneralStatistics.setLivingMalesAged30To49Number(livingMalesAged30To49Number);
			provinceGeneralStatistics.setLivingFemalesAged30To49Number(livingFemalesAged30To49Number);
			provinceGeneralStatistics.setLivingMalesAgedOver49Number(livingMalesAgedOver49Number);
			provinceGeneralStatistics.setLivingFemalesAgedOver49Number(livingFemalesAgedOver49Number);
			provinceGeneralStatistics.setLivingAuthorizedBirthRecordsNumber(livingAuthorizedBirthRecordsNumber);
			provinceGeneralStatistics.setLivingPeopleWithCardsNumber(livingPeopleWithCardsNumber);
			provinceGeneralStatistics.setLivingMalesAgedUnder18WithCardsNumber(livingMalesAgedUnder18WithCardsNumber);
			provinceGeneralStatistics
					.setLivingFemalesAgedUnder18WithCardsNumber(livingFemalesAgedUnder18WithCardsNumber);
			provinceGeneralStatistics.setLivingFemalesAged18To29WithCardsNumber(livingFemalesAged18To29WithCardsNumber);
			provinceGeneralStatistics.setLivingMalesAged18To29WithCardsNumber(livingMalesAged18To29WithCardsNumber);
			provinceGeneralStatistics.setLivingFemalesAged30To49WithCardsNumber(livingFemalesAged30To49WithCardsNumber);
			provinceGeneralStatistics.setLivingMalesAged30To49WithCardsNumber(livingMalesAged30To49WithCardsNumber);
			provinceGeneralStatistics.setLivingFemalesAgedOver49WithCardsNumber(livingFemalesAgedOver49WithCardsNumber);
			provinceGeneralStatistics.setLivingMalesAgedOver49WithCardsNumber(livingMalesAgedOver49WithCardsNumber);
			provinceGeneralStatisticsList.add(provinceGeneralStatistics);
		}
		return provinceGeneralStatisticsList;
	}
}
