package systems.archide.core.workflows;

import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import systems.archide.core.pojo.MtlbStatus;
import systems.archide.core.pojo.RvfourBirthMtlb;
import systems.archide.core.repository.IdcoreCustomQueries;
import systems.archide.core.repository.RvfourBirthMtlbAttachmentRepository;
import systems.archide.core.repository.RvfourBirthMtlbRepository;

@Component
public class RvFourBirthMtlbInitiated {
	
	private static final Logger log = LoggerFactory.getLogger(RvFourBirthMtlbInitiated.class);
	
	@Autowired
	RvfourBirthMtlbRepository rvfourBirthMtlbRepository;
	
	@Autowired
	RvfourBirthMtlbAttachmentRepository rvfourBirthMtlbAttachmentRepository;
	
	@Autowired
	IdcoreCustomQueries queries;
	
	@Scheduled(fixedDelay = 5000)
	public void fromInitiatedStatus() {
		List<RvfourBirthMtlb> mtlbList = queries.getRandomRv4BirthMtlbByMtlbStatus(MtlbStatus.INITIATED.getValue());
		if (mtlbList.size() > 0) {
			AtomicInteger completed = new AtomicInteger();
			mtlbList.parallelStream().forEach((mtlb) -> { //parallel exec for concurrent list
				if (!mtlb.isDraft()) {
					if (mtlb.getNumberOfAttachments() == null || mtlb.getNumberOfAttachments() == 0) {
						mtlb.setMtlbStatus(MtlbStatus.SUBMITTED);
						rvfourBirthMtlbRepository.update(mtlb);
					} else {
						long received = rvfourBirthMtlbAttachmentRepository
								.count("rvfour_birth_mtlb_id = " + mtlb.getId());
						if (received>=mtlb.getNumberOfAttachments()) {
							mtlb.setMtlbStatus(MtlbStatus.SUBMITTED);
							rvfourBirthMtlbRepository.update(mtlb);
							completed.getAndIncrement();
						}
					}
				}
			});
			if (completed.get() >0)
				log.info("{} Rv4 Birth applications completed transmission", completed);
		}
		else log.info("no newly initiated Rv4 Birth applications found");
	}

}
