package systems.archide.core.workflows;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.core.JsonProcessingException;

import systems.archide.core.pojo.ArchideMtlbAnnex;
import systems.archide.core.pojo.ArchideMtlbAnnexLogEntry;
import systems.archide.core.pojo.ArchideMtlbAnnexTag;
import systems.archide.core.pojo.ArchideMtlbAnnexUtil;
import systems.archide.core.pojo.CivilStatusMtlb;
import systems.archide.core.pojo.CivilStatusMtlbNote;
import systems.archide.core.pojo.CivilStatusMtlbTag;
import systems.archide.core.pojo.DeathRecord;
import systems.archide.core.pojo.DeathRecordNote;
import systems.archide.core.pojo.DeathRecordTag;
import systems.archide.core.pojo.Face;
import systems.archide.core.pojo.GlobalState;
import systems.archide.core.pojo.MtlbStatus;
import systems.archide.core.pojo.MtlbType;
import systems.archide.core.pojo.RvfourBirthMtlb;
import systems.archide.core.pojo.RvfourDeathMtlb;
import systems.archide.core.pojo.RvfourDeathMtlbTag;
import systems.archide.core.pojo.VitalRecord;
import systems.archide.core.repository.CivilStatusMtlbNoteRepository;
import systems.archide.core.repository.CivilStatusMtlbRepository;
import systems.archide.core.repository.CivilStatusMtlbTagRepository;
import systems.archide.core.repository.DeathRecordNoteRepository;
import systems.archide.core.repository.DeathRecordRepository;
import systems.archide.core.repository.DeathRecordTagRepository;
import systems.archide.core.repository.FaceRepository;
import systems.archide.core.repository.GlobalStateRepository;
import systems.archide.core.repository.IdcoreCustomQueries;
import systems.archide.core.repository.RvfourBirthMtlbRepository;
import systems.archide.core.repository.RvfourDeathMtlbRepository;
import systems.archide.core.repository.RvfourDeathMtlbTagRepository;
import systems.archide.core.repository.VitalRecordRepository;
import systems.archide.core.workflows.utils.CustomErrorCodes;
import systems.archide.core.workflows.utils.Utils;

@Component
public class RvfourDeathMtlbSubmitted {
	
	private static final Logger log = LoggerFactory.getLogger(RvFourBirthMtlbSubmitted.class);
	
	@Autowired
	RvfourDeathMtlbRepository rvfourDeathMtlbRepository;
	
	@Autowired
	GlobalStateRepository globalStateRepository;
	
	@Autowired
	VitalRecordRepository vitalRecordRepository;
	
	@Autowired
	RvfourDeathMtlbTagRepository rvfourDeathMtlbTagRepository;
	
	@Autowired
	DeathRecordNoteRepository deathRecordNoteRepository;
	
	@Autowired
	DeathRecordRepository deathRecordRepository;
	
	@Autowired
	RvfourBirthMtlbRepository rvfourBirthMtlbRepository;
	
	@Autowired
	DeathRecordTagRepository deathRecordTagRepository;
	
	@Autowired
	CivilStatusMtlbRepository civilStatusMtlbRepository;
	
	@Autowired
	CivilStatusMtlbTagRepository civilStatusMtlbTagRepository;
	
	@Autowired
	CivilStatusMtlbNoteRepository civilStatusMtlbNoteRepository;
	
	@Autowired
	FaceRepository faceRepository;
	
	@Autowired
	IdcoreCustomQueries queries;
	
	@Autowired
	Utils utils;
	
	@Scheduled(fixedDelay = 5000)
	public void fromSubmittedStatus() {
		GlobalState migrationState = globalStateRepository.findByKey("migration");
		List<RvfourDeathMtlb> mtlbList = queries.getRandomRv4DeathMtlbByMtlbStatus(MtlbStatus.SUBMITTED.getValue());
		if (mtlbList.size() > 0) {
			log.info("Read {} newly submitted rv4 death applications", mtlbList.size());
			for (RvfourDeathMtlb mtlb : mtlbList) {
				if (mtlb.getVitalRecordId() != null) {
					switch(mtlb.getMtlbType()) {
					case RV4_DEATH_RECORD:
						createDeathRecord(mtlb, migrationState);
						break;
					case RV4_DEATH_APPLICATION:
						createDeathApplication(mtlb, migrationState);
						break;
					case RV4_DEATH_RECORD_TRASH:
						createDeathRecordTrash(mtlb);
						break;
					}
					rvfourDeathMtlbRepository.save(mtlb);
				}
			}
		}
	}

	private void createDeathRecordTrash(RvfourDeathMtlb mtlb) {
		// TODO Auto-generated method stub
		
	}

	private void createDeathApplication(RvfourDeathMtlb mtlb, GlobalState migrationState) {
		CivilStatusMtlb cMtlb = new CivilStatusMtlb();
		ArrayList<CivilStatusMtlbTag> tags = new ArrayList<CivilStatusMtlbTag>();
		ArrayList<CivilStatusMtlbNote> notes = new ArrayList<CivilStatusMtlbNote>();
		int recordStatus = utils.checkVitalRecordStatus(mtlb.getVitalRecordId(), migrationState);
		cMtlb.setVitalRecordId(mtlb.getVitalRecordId());
		cMtlb.setMtlbStatus(MtlbStatus.SUBMITTED);
		VitalRecord vr = null;
		if (recordStatus == Utils.FOUND) {
			vr = vitalRecordRepository.findById(mtlb.getVitalRecordId());
		}
		else if (recordStatus == Utils.UNKNOWN || recordStatus == Utils.FOUND_BUT_VITAL_RECORD_NOT_CREATED_YET)
			return;
		else if (recordStatus == Utils.CONFIRMED_MISSING) {
			CivilStatusMtlbTag tag = null;
			tag = new CivilStatusMtlbTag();
			tag.setContent(CustomErrorCodes.VITAL_RECORD_ID_MISSING);
			tag.setType(Utils.WARNING_TAG_TYPE);
			tags.add(tag);
			cMtlb.setVitalRecordId(null);
		}
		else if (recordStatus == Utils.FOUND_AS_FAMILY_TREE_RECORD) {
			CivilStatusMtlbTag tag = null;
			tag = new CivilStatusMtlbTag();
			tag.setKey(CustomErrorCodes.RECORD_FOUND_IN_FAMILY_TREE);
			tag.setContent(mtlb.getVitalRecordId().toString());
			tag.setType(Utils.WARNING_TAG_TYPE);
			tags.add(tag);
			RvfourBirthMtlb rvfourBirthMtlb = rvfourBirthMtlbRepository.findByVitalRecordId(mtlb.getVitalRecordId());
			if (rvfourBirthMtlb != null) {
				cMtlb.setFirstname(rvfourBirthMtlb.getFirstname());
				cMtlb.setSecondname(rvfourBirthMtlb.getSecondname());
				cMtlb.setThirdname(rvfourBirthMtlb.getThirdname());
				cMtlb.setFourthname(rvfourBirthMtlb.getFourthname());
				cMtlb.setGender(rvfourBirthMtlb.getGender());
				cMtlb.setMaritalStatus(rvfourBirthMtlb.getMaritalStatus());
				cMtlb.setReportedFatherName(rvfourBirthMtlb.getReportedFatherName());
				cMtlb.setReportedMotherName(rvfourBirthMtlb.getReportedMotherName());
			}
			cMtlb.setVitalRecordId(null);
		}
		if (vr != null) {
			cMtlb.setFirstname(vr.getFirstname());
			cMtlb.setSecondname(vr.getSecondname());
			cMtlb.setThirdname(vr.getThirdname());
			cMtlb.setFourthname(vr.getFourthname());
			cMtlb.setGender(vr.getGender());
			cMtlb.setMaritalStatus(vr.getMaritalStatus());
			if (vr.getFaceId() != null) {
				Face face = faceRepository.findById(vr.getFaceId());
				cMtlb.setFace(face.getImage());
			}
			cMtlb.setReportedFatherName(vr.getReportedFatherName());
			cMtlb.setReportedMotherName(vr.getReportedMotherName());
		}
		cMtlb.setMtlbType(MtlbType.DEATH_REGISTRATION);
		cMtlb.setVitalRecordId(mtlb.getVitalRecordId());
		cMtlb.setTimeOfDeath(mtlb.getTimeOfDeath());
		cMtlb.setDateOfDeath(mtlb.getDateOfDeath());
		cMtlb.setDeclarantId(mtlb.getDeclarantId());
		cMtlb.setDeathPlace(mtlb.getDeathLocation());
		Long id = civilStatusMtlbRepository.insert(cMtlb);
		if (id != null)
			mtlb.setMtlbStatus(MtlbStatus.ARCHIVED);
		ArchideMtlbAnnex annex = null;
		try {
			annex = ArchideMtlbAnnexUtil.deserialize(mtlb.getAnnex());
		} catch (JsonProcessingException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		if (annex.getTags() != null)
			for (ArchideMtlbAnnexTag tag : annex.getTags()) {
				CivilStatusMtlbTag cTag = new CivilStatusMtlbTag();
				cTag.setCivilStatusMtlbId(id);
				cTag.setContent(tag.getValue());
				cTag.setType(tag.getType());
				cTag.setKey(tag.getName());
				tags.add(cTag);
			}
		for (CivilStatusMtlbTag tag: tags) {
			tag.setCivilStatusMtlbId(id);
			civilStatusMtlbTagRepository.insert(tag);
		}
		if (annex.getNotes() != null)
			for (ArchideMtlbAnnexLogEntry note : annex.getNotes()) {
				CivilStatusMtlbNote cNote = new CivilStatusMtlbNote();
				cNote.setContent(note.getContent());
				cNote.setAddedBy(note.getAuthor());
				cNote.setTimestamp(note.getTimestamp());
				notes.add(cNote);
			}
		for (CivilStatusMtlbNote note: notes) {
			note.setCivilStatusMtlbId(id);
			civilStatusMtlbNoteRepository.insert(note);
		}
	}

	private void createDeathRecord(RvfourDeathMtlb mtlb, GlobalState migrationState) {
		DeathRecord dr = new DeathRecord();
		ArrayList<DeathRecordTag> tags = new ArrayList<DeathRecordTag>();
		int recordStatus = utils.checkVitalRecordStatus(mtlb.getVitalRecordId(), migrationState);
		dr.setVitalRecordId(mtlb.getVitalRecordId());
		VitalRecord vr = null;
		if (recordStatus == Utils.FOUND) {
			vr = vitalRecordRepository.findById(mtlb.getVitalRecordId());
		}
		else if (recordStatus == Utils.UNKNOWN || recordStatus == Utils.FOUND_BUT_VITAL_RECORD_NOT_CREATED_YET)
			return;
		else if (recordStatus == Utils.CONFIRMED_MISSING) {
			DeathRecordTag tag = null;
			tag = new DeathRecordTag();
			tag.setDeathRecordId(mtlb.getDeathRecordId());
			tag.setContent(CustomErrorCodes.VITAL_RECORD_ID_MISSING);
			tag.setType(Utils.WARNING_TAG_TYPE);
			tags.add(tag);
			dr.setVitalRecordId(null);
		}
		else if (recordStatus == Utils.FOUND_AS_FAMILY_TREE_RECORD) {
			DeathRecordTag tag = null;
			tag = new DeathRecordTag();
			tag.setDeathRecordId(mtlb.getDeathRecordId());
			tag.setKey(CustomErrorCodes.RECORD_FOUND_IN_FAMILY_TREE);
			tag.setContent(mtlb.getVitalRecordId().toString());
			tag.setType(Utils.WARNING_TAG_TYPE);
			tags.add(tag);
			RvfourBirthMtlb rvfourBirthMtlb = rvfourBirthMtlbRepository.findByVitalRecordId(mtlb.getVitalRecordId());
			if (rvfourBirthMtlb != null) {
				dr.setFirstname(rvfourBirthMtlb.getFirstname());
				dr.setSecondname(rvfourBirthMtlb.getSecondname());
				dr.setThirdname(rvfourBirthMtlb.getThirdname());
				dr.setFourthname(rvfourBirthMtlb.getFourthname());
				dr.setGender(rvfourBirthMtlb.getGender());
				dr.setMaritalStatus(rvfourBirthMtlb.getMaritalStatus());
				dr.setReportedFatherName(rvfourBirthMtlb.getReportedFatherName());
				dr.setReportedMotherName(rvfourBirthMtlb.getReportedMotherName());
			}
			dr.setVitalRecordId(null);
		}
		if (vr != null) {
			dr.setFirstname(vr.getFirstname());
			dr.setSecondname(vr.getSecondname());
			dr.setThirdname(vr.getThirdname());
			dr.setFourthname(vr.getFourthname());
			dr.setGender(vr.getGender());
			dr.setMaritalStatus(vr.getMaritalStatus());
			dr.setFaceId(vr.getFaceId());
			dr.setReportedFatherName(vr.getReportedFatherName());
			dr.setReportedMotherName(vr.getReportedMotherName());
		}
		dr.setId(mtlb.getDeathRecordId());
		dr.setTimeOfDeath(mtlb.getTimeOfDeath());
		dr.setDateOfDeath(mtlb.getDateOfDeath());
		dr.setDraft(false);
		dr.setDeathLocation(mtlb.getDeathLocation());
		dr.setDeathPlace(mtlb.getDeathLocation());
		ArchideMtlbAnnex annex = null;
		try {
			annex = ArchideMtlbAnnexUtil.deserialize(mtlb.getAnnex());
		} catch (JsonProcessingException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		if (annex.getTags() != null)
			for (ArchideMtlbAnnexTag tag : annex.getTags()) {
				DeathRecordTag drTag = new DeathRecordTag();
				drTag.setDeathRecordId(mtlb.getDeathRecordId());
				drTag.setContent(tag.getValue());
				drTag.setType(tag.getType());
				drTag.setKey(tag.getName());
				tags.add(drTag);
			}
		
		
		List<RvfourDeathMtlbTag> mtlbTagList = rvfourDeathMtlbTagRepository.findByRvfourDeathMtlbId(mtlb.getId());
		for (RvfourDeathMtlbTag tag : mtlbTagList) {
			DeathRecordTag vt = new DeathRecordTag();
			vt.setDeathRecordId(mtlb.getDeathRecordId());
			vt.setContent(tag.getContent());
			vt.setType(tag.getType());
			vt.setKey(tag.getKey());
			tags.add(vt);
		}
		ArrayList<DeathRecordNote> notes = new ArrayList<>();
		for (ArchideMtlbAnnexLogEntry note : annex.getNotes()) {
			DeathRecordNote vn = new DeathRecordNote();
			vn.setAddedBy(note.getAuthor());
			vn.setContent(note.getContent());
			vn.setTimestamp(note.getTimestamp());
			vn.setDeathRecordId(mtlb.getDeathRecordId());
			notes.add(vn);
		}
		try {
			for (DeathRecordTag tag : tags)
				deathRecordTagRepository.insert(tag);
			Long id = deathRecordRepository.insert(dr);
			if (id != null)
				mtlb.setMtlbStatus(MtlbStatus.ARCHIVED);
			for (DeathRecordNote note : notes)
				deathRecordNoteRepository.insert(note);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
}
