package systems.archide.core.workflows;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.core.JsonProcessingException;

import systems.archide.core.pojo.ArchideMtlbAnnex;
import systems.archide.core.pojo.ArchideMtlbAnnexLogEntry;
import systems.archide.core.pojo.ArchideMtlbAnnexTag;
import systems.archide.core.pojo.ArchideMtlbAnnexUtil;
import systems.archide.core.pojo.GlobalState;
import systems.archide.core.pojo.MtlbStatus;
import systems.archide.core.pojo.MtlbType;
import systems.archide.core.pojo.RvfourDivorceMtlb;
import systems.archide.core.pojo.UnionMtlb;
import systems.archide.core.pojo.UnionMtlbTag;
import systems.archide.core.pojo.UnionRecord;
import systems.archide.core.pojo.UnionRecordMention;
import systems.archide.core.pojo.UnionRecordTag;
import systems.archide.core.repository.GlobalStateRepository;
import systems.archide.core.repository.IdcoreCustomQueries;
import systems.archide.core.repository.RvfourDivorceMtlbRepository;
import systems.archide.core.repository.UnionMtlbRepository;
import systems.archide.core.repository.UnionMtlbTagRepository;
import systems.archide.core.repository.UnionRecordMentionRepository;
import systems.archide.core.repository.UnionRecordRepository;
import systems.archide.core.repository.UnionRecordTagRepository;
import systems.archide.core.workflows.utils.CustomErrorCodes;
import systems.archide.core.workflows.utils.Utils;

@Component
public class RvfourDivorceMtlbSubmitted {

	private static final Logger log = LoggerFactory.getLogger(RvfourMarriageMtlbSubmitted.class);

	@Autowired
	GlobalStateRepository globalStateRepository;
	
	@Autowired
	RvfourDivorceMtlbRepository rvfourDivorceMtlbRepository;
	
	@Autowired
	UnionRecordRepository unionRecordRepository;
	
	@Autowired
	UnionRecordMentionRepository unionRecordMentionRepository;
	
	@Autowired
	UnionRecordTagRepository unionRecordTagRepository;
	
	@Autowired
	UnionMtlbRepository unionMtlbRepository;
	
	@Autowired
	UnionMtlbTagRepository unionMtlbTagRepository;
	
	@Autowired
	IdcoreCustomQueries queries;
	
	@Autowired
	Utils utils;
	
	@Scheduled(fixedDelay = 5000)
	public void fromSubmittedStatus() {
		GlobalState migrationState = globalStateRepository.findByKey("migration");
		List<RvfourDivorceMtlb> mtlbList = queries.getRandomRv4DivorceMtlbByMtlbStatus(MtlbStatus.SUBMITTED.getValue());
		if (mtlbList.size() > 0) {
			log.info("Read {} newly submitted rv4 divorce applications", mtlbList.size());
			for (RvfourDivorceMtlb mtlb : mtlbList) {
				switch (mtlb.getMtlbType()) {
				case RV4_UNION_RECORD:
					createUnionRecord(mtlb, migrationState);
					break;
				case RV4_DIVORCE_APPLICATION:
					createDivorceApplication(mtlb, migrationState);
					break;
				case RV4_UNION_RECORD_TRASH:
					createUnionRecordTrash(mtlb);
					break;
				}
				rvfourDivorceMtlbRepository.save(mtlb);

			}
		}
	}

	private void createUnionRecordTrash(RvfourDivorceMtlb mtlb) {
		// TODO Auto-generated method stub
		
	}

	private void createDivorceApplication(RvfourDivorceMtlb mtlb, GlobalState migrationState) {
		UnionMtlb um = new UnionMtlb();
		ArrayList<UnionMtlbTag> tags = new ArrayList<UnionMtlbTag>();
		um.setDraft(false);
		um.setMtlbStatus(MtlbStatus.SUBMITTED);
		um.setMtlbType(MtlbType.DIVORCE);
		int unionRecordStatus = utils.checkVitalRecordStatus(mtlb.getUnionRecordId(), migrationState);
		UnionRecord ur = null;
		if (unionRecordStatus == Utils.FOUND)
			ur = unionRecordRepository.findById(mtlb.getUnionRecordId());
		else if (unionRecordStatus == Utils.UNKNOWN || unionRecordStatus == Utils.FOUND_BUT_UNION_RECORD_NOT_CREATED_YET)
			return;
		else if (unionRecordStatus == Utils.CONFIRMED_MISSING) {
			UnionMtlbTag tag = null;
			tag = new UnionMtlbTag();
			tag.setContent(CustomErrorCodes.UNION_RECORD_MISSING);
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
			um.setMtlbStatus(MtlbStatus.MANUAL_CHECK);
		}
		um.setDateOfEffect(mtlb.getDateOfEffect());
		um.setDateOfOrder(mtlb.getDateOfOrder());
		if (ur != null) {
			um.setPartner1VitalRecordId(ur.getPartner1VitalRecordId());
			um.setPartner2VitalRecordId(ur.getPartner2VitalRecordId());
			um.setPartner1Firstname(ur.getPartner1Firstname());
			um.setPartner2Firstname(ur.getPartner2Firstname());
			um.setPartner1Surname(ur.getPartner1Surname());
			um.setPartner2Surname(ur.getPartner2Surname());
		}
		ArchideMtlbAnnex annex = null;
		try {
			if (mtlb.getAnnex() != null)
				annex = ArchideMtlbAnnexUtil.deserialize(mtlb.getAnnex());
		} catch (JsonProcessingException e1) {
			e1.printStackTrace();
		}
		
		if (annex != null) {
			for (ArchideMtlbAnnexTag tag : annex.getTags()) {
				UnionMtlbTag umTag = new UnionMtlbTag();
				umTag.setContent(tag.getValue());
				umTag.setType(tag.getType());
				umTag.setKey(tag.getName());
				tags.add(umTag);
			}
			if (annex.getMentions() != null)
				for (ArchideMtlbAnnexLogEntry mention : annex.getMentions()) {
					if ("Annulment Order".equals(mention.getAuthor()))
						um.setAnnullmentOrder(mention.getContent());
				}
		}
		try {
			Long id = unionMtlbRepository.insert(um);
			for (UnionMtlbTag tag : tags) {
				tag.setUnionMtlbId(id);
				unionMtlbTagRepository.insert(tag);
			}
			mtlb.setMtlbStatus(MtlbStatus.ARCHIVED);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}

	private void createUnionRecord(RvfourDivorceMtlb mtlb, GlobalState migrationState) {
		ArrayList<UnionRecordTag> tags = new ArrayList<UnionRecordTag>();
		int unionRecordStatus = utils.checkVitalRecordStatus(mtlb.getUnionRecordId(), migrationState);
		UnionRecord ur = null;
		if (unionRecordStatus == Utils.FOUND)
			ur = unionRecordRepository.findById(mtlb.getUnionRecordId());
		else if (unionRecordStatus == Utils.UNKNOWN || unionRecordStatus == Utils.FOUND_BUT_UNION_RECORD_NOT_CREATED_YET)
			return;
		else if (unionRecordStatus == Utils.CONFIRMED_MISSING) {
			ur = new UnionRecord();
			UnionRecordTag tag = null;
			tag = new UnionRecordTag();
			tag.setUnionRecordId(mtlb.getUnionRecordId());
			tag.setContent(CustomErrorCodes.UNION_RECORD_MISSING);
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
			ur.setId(mtlb.getUnionRecordId());
			ur.setDateOfEffect(mtlb.getDateOfEffect());
			ur.setDateOfOrder(mtlb.getDateOfOrder());
		}
		ArchideMtlbAnnex annex = null;
		ArrayList<UnionRecordMention> mentions = new ArrayList<>();
		if (mtlb.getAnnex() != null)
			try {
				annex = ArchideMtlbAnnexUtil.deserialize(mtlb.getAnnex());
			} catch (JsonProcessingException e1) {
				e1.printStackTrace();
			}
		if (annex != null) {
			if (annex.getTags() != null)
				for (ArchideMtlbAnnexTag tag : annex.getTags()) {
					UnionRecordTag urTag = new UnionRecordTag();
					urTag.setUnionRecordId(mtlb.getUnionRecordId());
					urTag.setContent(tag.getValue());
					urTag.setType(tag.getType());
					urTag.setKey(tag.getName());
					tags.add(urTag);
				}
			for (ArchideMtlbAnnexLogEntry mention : annex.getMentions()) {
				UnionRecordMention vm = new UnionRecordMention();
				vm.setAddedBy(mention.getAuthor());
				vm.setContent(mention.getContent());
				vm.setTimestamp(mention.getTimestamp());
				vm.setUnionRecordId(mtlb.getUnionRecordId());
				mentions.add(vm);
				if ("Annulment Order".equals(mention.getAuthor()))
					ur.setAnnulmentOrder(mention.getContent());
			}
		}
		try {
			unionRecordRepository.save(ur);
			for (UnionRecordMention mention : mentions)
				unionRecordMentionRepository.insert(mention);
			for (UnionRecordTag tag : tags)
				unionRecordTagRepository.insert(tag);
			mtlb.setMtlbStatus(MtlbStatus.ARCHIVED);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}

}
