package systems.archide.core.workflows;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.core.JsonProcessingException;

import systems.archide.core.pojo.ArchideMtlbAnnex;
import systems.archide.core.pojo.ArchideMtlbAnnexLogEntry;
import systems.archide.core.pojo.ArchideMtlbAnnexTag;
import systems.archide.core.pojo.ArchideMtlbAnnexUtil;
import systems.archide.core.pojo.GlobalState;
import systems.archide.core.pojo.MtlbStatus;
import systems.archide.core.pojo.MtlbType;
import systems.archide.core.pojo.RvfourBirthMtlb;
import systems.archide.core.pojo.RvfourMarriageMtlb;
import systems.archide.core.pojo.UnionMtlb;
import systems.archide.core.pojo.UnionMtlbTag;
import systems.archide.core.pojo.UnionRecord;
import systems.archide.core.pojo.UnionRecordMention;
import systems.archide.core.pojo.UnionRecordTag;
import systems.archide.core.pojo.VitalRecord;
import systems.archide.core.repository.GlobalStateRepository;
import systems.archide.core.repository.IdcoreCustomQueries;
import systems.archide.core.repository.RvfourBirthMtlbRepository;
import systems.archide.core.repository.RvfourMarriageMtlbRepository;
import systems.archide.core.repository.UnionMtlbRepository;
import systems.archide.core.repository.UnionMtlbTagRepository;
import systems.archide.core.repository.UnionRecordMentionRepository;
import systems.archide.core.repository.UnionRecordRepository;
import systems.archide.core.repository.UnionRecordTagRepository;
import systems.archide.core.repository.VitalRecordRepository;
import systems.archide.core.workflows.utils.CustomErrorCodes;
import systems.archide.core.workflows.utils.Utils;

@Component
public class RvfourMarriageMtlbSubmitted {

	private static final Logger log = LoggerFactory.getLogger(RvfourMarriageMtlbSubmitted.class);

	@Autowired
	GlobalStateRepository globalStateRepository;

	@Autowired
	RvfourMarriageMtlbRepository rvfourMarriageMtlbRepository;
	
	@Autowired
	VitalRecordRepository vitalRecordRepository;
	
	@Autowired
	UnionRecordTagRepository unionRecordTagRepository;
	
	@Autowired
	UnionRecordRepository unionRecordRepository;
	
	@Autowired
	UnionRecordMentionRepository unionRecordMentionRepository;
	
	@Autowired
	RvfourBirthMtlbRepository rvfourBirthMtlbRepository;
	
	@Autowired
	UnionMtlbRepository unionMtlbRepository;
	
	@Autowired
	UnionMtlbTagRepository unionMtlbTagRepository;
	
	@Autowired
	IdcoreCustomQueries queries;
	
	@Autowired
	Utils utils;

	@Scheduled(fixedDelay = 5000)
	public void fromSubmittedStatus() {
		GlobalState migrationState = globalStateRepository.findByKey("migration");
		List<RvfourMarriageMtlb> mtlbList = queries.getRandomRv4MarriageMtlbByMtlbStatus(MtlbStatus.SUBMITTED.getValue());
		if (mtlbList.size() > 0) {
			log.info("Read {} newly submitted rv4 marriage applications", mtlbList.size());
			for (RvfourMarriageMtlb mtlb : mtlbList) {
				switch (mtlb.getMtlbType()) {
				case RV4_UNION_RECORD:
					createUnionRecord(mtlb, migrationState);
					break;
				case RV4_MARRIAGE_APPLICATION:
					createMarriageApplication(mtlb, migrationState);
					break;
				case RV4_UNION_RECORD_TRASH:
					createUnionRecordTrash(mtlb);
					break;
				}
				rvfourMarriageMtlbRepository.save(mtlb);

			}
		}
	}

	private void createUnionRecordTrash(RvfourMarriageMtlb mtlb) {
		// TODO Auto-generated method stub
		
	}

	private void createMarriageApplication(RvfourMarriageMtlb mtlb, GlobalState migrationState) {
		UnionMtlb um = new UnionMtlb();
		ArrayList<UnionMtlbTag> tags = new ArrayList<UnionMtlbTag>();
		um.setDraft(false);
		um.setMtlbStatus(MtlbStatus.SUBMITTED);
		um.setMtlbType(MtlbType.MARRIAGE);
		int hrecordStatus = utils.checkVitalRecordStatus(mtlb.getHusbandId(), migrationState);
		VitalRecord hvr = null;
		if (hrecordStatus == Utils.FOUND) {
			hvr = vitalRecordRepository.findById(mtlb.getHusbandId());
			um.setPartner1Firstname(hvr.getFirstname());
			um.setPartner1Surname(hvr.getFourthname());
		}
		else if (hrecordStatus == Utils.UNKNOWN || hrecordStatus == Utils.FOUND_BUT_VITAL_RECORD_NOT_CREATED_YET)
			return;
		else if (hrecordStatus == Utils.CONFIRMED_MISSING) {
			UnionMtlbTag tag = null;
			tag = new UnionMtlbTag();
			tag.setUnionMtlbId(mtlb.getId());
			tag.setContent(CustomErrorCodes.HUSBAND_RECORD_ID_MISSING);
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
			um.setMtlbStatus(MtlbStatus.MANUAL_CHECK);
		}
		else if (hrecordStatus == Utils.FOUND_AS_FAMILY_TREE_RECORD) {
			UnionMtlbTag tag = null;
			tag = new UnionMtlbTag();
			tag.setUnionMtlbId(mtlb.getId());
			tag.setKey(CustomErrorCodes.HUSBAND_RECORD_FOUND_IN_FAMILY_TREE);
			tag.setContent(mtlb.getHusbandId().toString());
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
			RvfourBirthMtlb rvfourBirthMtlb = rvfourBirthMtlbRepository.findByVitalRecordId(mtlb.getHusbandId());
			if (rvfourBirthMtlb != null) {
				um.setPartner1Firstname(rvfourBirthMtlb.getFirstname());
				um.setPartner1Surname(rvfourBirthMtlb.getFourthname());
			}
			um.setMtlbStatus(MtlbStatus.MANUAL_CHECK);
		}
		int wifeRecordStatus = utils.checkVitalRecordStatus(mtlb.getWifeId(), migrationState);
		VitalRecord wvr = null;
		if (wifeRecordStatus == Utils.FOUND) {
			wvr = vitalRecordRepository.findById(mtlb.getWifeId());
			um.setPartner2Firstname(wvr.getFirstname());
			um.setPartner2Surname(wvr.getFourthname());
		}
		else if (wifeRecordStatus == Utils.UNKNOWN || wifeRecordStatus == Utils.FOUND_BUT_VITAL_RECORD_NOT_CREATED_YET)
			return;
		else if (wifeRecordStatus == Utils.CONFIRMED_MISSING) {
			UnionMtlbTag tag = null;
			tag = new UnionMtlbTag();
			tag.setUnionMtlbId(mtlb.getId());
			tag.setContent(CustomErrorCodes.WIFE_RECORD_ID_MISSING);
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
			um.setMtlbStatus(MtlbStatus.MANUAL_CHECK);
		}
		else if (wifeRecordStatus == Utils.FOUND_AS_FAMILY_TREE_RECORD) {
			UnionMtlbTag tag = null;
			tag = new UnionMtlbTag();
			tag.setUnionMtlbId(mtlb.getId());
			tag.setKey(CustomErrorCodes.Wife_RECORD_FOUND_IN_FAMILY_TREE);
			tag.setContent(mtlb.getWifeId().toString());
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
			RvfourBirthMtlb rvfourBirthMtlb = rvfourBirthMtlbRepository.findByVitalRecordId(mtlb.getWifeId());
			if (rvfourBirthMtlb != null) {
				um.setPartner2Firstname(rvfourBirthMtlb.getFirstname());
				um.setPartner2Surname(rvfourBirthMtlb.getFourthname());
			}
			um.setMtlbStatus(MtlbStatus.MANUAL_CHECK);
		}
		int witness1RecordStatus = utils.checkVitalRecordStatus(mtlb.getWitness1Id(), migrationState);
		if (witness1RecordStatus == Utils.UNKNOWN || witness1RecordStatus == Utils.FOUND_BUT_VITAL_RECORD_NOT_CREATED_YET)
			return;
		else if (witness1RecordStatus == Utils.CONFIRMED_MISSING) {
			UnionMtlbTag tag = null;
			tag = new UnionMtlbTag();
			tag.setUnionMtlbId(mtlb.getId());
			tag.setContent(CustomErrorCodes.WITNESS1_RECORD_ID_MISSING);
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
			um.setMtlbStatus(MtlbStatus.MANUAL_CHECK);
		}
		else if (witness1RecordStatus == Utils.FOUND_AS_FAMILY_TREE_RECORD) {
			UnionMtlbTag tag = null;
			tag = new UnionMtlbTag();
			tag.setUnionMtlbId(mtlb.getId());
			tag.setKey(CustomErrorCodes.WITNESS1_RECORD_FOUND_IN_FAMILY_TREE);
			tag.setContent(mtlb.getWitness1Id().toString());
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
			um.setMtlbStatus(MtlbStatus.MANUAL_CHECK);
		}
		int witness2RecordStatus = utils.checkVitalRecordStatus(mtlb.getWitness2Id(), migrationState);
		if (witness2RecordStatus == Utils.UNKNOWN || witness2RecordStatus == Utils.FOUND_BUT_VITAL_RECORD_NOT_CREATED_YET)
			return;
		else if (witness2RecordStatus == Utils.CONFIRMED_MISSING) {
			UnionMtlbTag tag = null;
			tag = new UnionMtlbTag();
			tag.setUnionMtlbId(mtlb.getId());
			tag.setContent(CustomErrorCodes.WITNESS2_RECORD_ID_MISSING);
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
			um.setMtlbStatus(MtlbStatus.MANUAL_CHECK);
		}
		else if (witness2RecordStatus == Utils.FOUND_AS_FAMILY_TREE_RECORD) {
			UnionMtlbTag tag = null;
			tag = new UnionMtlbTag();
			tag.setUnionMtlbId(mtlb.getId());
			tag.setKey(CustomErrorCodes.WITNESS2_RECORD_FOUND_IN_FAMILY_TREE);
			tag.setContent(mtlb.getWitness2Id().toString());
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
			um.setMtlbStatus(MtlbStatus.MANUAL_CHECK);
		}
		um.setId(mtlb.getId());
		um.setAddress(mtlb.getMarriageLocation());
		um.setPlace(mtlb.getMarriagePlace());
		um.setPartner1VitalRecordId(mtlb.getHusbandId());
		um.setPartner2VitalRecordId(mtlb.getWifeId());
		um.setDeclaredDate(mtlb.getDateOfMarriage());
		um.setWitness1VitalRecordId(mtlb.getWitness1Id());
		um.setWitness2VitalRecordId(mtlb.getWitness2Id());
		um.setRites(mtlb.getMarriageRites());
		um.setTimeOfMarriage(mtlb.getTimeOfMarriage());
		if (hvr != null && wvr != null) {
			if (hvr.getGender() == 2 && wvr.getGender() == 1) {
				um.setPartner1VitalRecordId(wvr.getId());
				um.setPartner2VitalRecordId(hvr.getId());
			} else if (hvr.getGender() == wvr.getGender()) {
				UnionMtlbTag tag = new UnionMtlbTag();
				tag.setUnionMtlbId(mtlb.getId());
				tag.setContent(hvr.getId() + " : " + wvr.getId());
				tag.setType(Utils.ERROR_TAG_TYPE);
				tag.setKey(CustomErrorCodes.SAME_SEX_MARRIAGE);
				tags.add(tag);
				um.setMtlbStatus(MtlbStatus.MANUAL_CHECK);
			}
		}
		ArchideMtlbAnnex annex = null;
		try {
			annex = ArchideMtlbAnnexUtil.deserialize(mtlb.getAnnex());
		} catch (JsonProcessingException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		if (annex.getTags() != null)
			for (ArchideMtlbAnnexTag tag : annex.getTags()) {
				UnionMtlbTag umTag = new UnionMtlbTag();
				umTag.setUnionMtlbId(mtlb.getId());
				umTag.setContent(tag.getValue());
				umTag.setType(tag.getType());
				umTag.setKey(tag.getName());
				tags.add(umTag);
			}
		try {
			unionMtlbRepository.insert(um);
			for (UnionMtlbTag tag : tags)
				unionMtlbTagRepository.insert(tag);
			mtlb.setMtlbStatus(MtlbStatus.ARCHIVED);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void createUnionRecord(RvfourMarriageMtlb mtlb, GlobalState migrationState) {
		UnionRecord ur = new UnionRecord();
		ArrayList<UnionRecordTag> tags = new ArrayList<UnionRecordTag>();
		
		int hrecordStatus = utils.checkVitalRecordStatus(mtlb.getHusbandId(), migrationState);
		VitalRecord hvr = null;
		if (hrecordStatus == Utils.FOUND) {
			hvr = vitalRecordRepository.findById(mtlb.getHusbandId());
			ur.setPartner1Firstname(hvr.getFirstname());
			ur.setPartner1Surname(hvr.getFourthname());
		}
		else if (hrecordStatus == Utils.UNKNOWN || hrecordStatus == Utils.FOUND_BUT_VITAL_RECORD_NOT_CREATED_YET)
			return;
		else if (hrecordStatus == Utils.CONFIRMED_MISSING) {
			UnionRecordTag tag = null;
			tag = new UnionRecordTag();
			tag.setUnionRecordId(mtlb.getId());
			tag.setContent(CustomErrorCodes.HUSBAND_RECORD_ID_MISSING);
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
		}
		else if (hrecordStatus == Utils.FOUND_AS_FAMILY_TREE_RECORD) {
			UnionRecordTag tag = null;
			tag = new UnionRecordTag();
			tag.setUnionRecordId(mtlb.getId());
			tag.setKey(CustomErrorCodes.HUSBAND_RECORD_FOUND_IN_FAMILY_TREE);
			tag.setContent(mtlb.getHusbandId().toString());
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
			RvfourBirthMtlb rvfourBirthMtlb = rvfourBirthMtlbRepository.findByVitalRecordId(mtlb.getHusbandId());
			if (rvfourBirthMtlb != null) {
				ur.setPartner1Firstname(rvfourBirthMtlb.getFirstname());
				ur.setPartner1Surname(rvfourBirthMtlb.getFourthname());
			}
		}
		int wifeRecordStatus = utils.checkVitalRecordStatus(mtlb.getWifeId(), migrationState);
		VitalRecord wvr = null;
		if (wifeRecordStatus == Utils.FOUND) {
			wvr = vitalRecordRepository.findById(mtlb.getWifeId());
			ur.setPartner2Firstname(wvr.getFirstname());
			ur.setPartner2Surname(wvr.getFourthname());
		}
		else if (wifeRecordStatus == Utils.UNKNOWN || wifeRecordStatus == Utils.FOUND_BUT_VITAL_RECORD_NOT_CREATED_YET)
			return;
		else if (wifeRecordStatus == Utils.CONFIRMED_MISSING) {
			UnionRecordTag tag = null;
			tag = new UnionRecordTag();
			tag.setUnionRecordId(mtlb.getId());
			tag.setContent(CustomErrorCodes.WIFE_RECORD_ID_MISSING);
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
		}
		else if (wifeRecordStatus == Utils.FOUND_AS_FAMILY_TREE_RECORD) {
			UnionRecordTag tag = null;
			tag = new UnionRecordTag();
			tag.setUnionRecordId(mtlb.getId());
			tag.setKey(CustomErrorCodes.Wife_RECORD_FOUND_IN_FAMILY_TREE);
			tag.setContent(mtlb.getWifeId().toString());
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
			RvfourBirthMtlb rvfourBirthMtlb = rvfourBirthMtlbRepository.findByVitalRecordId(mtlb.getWifeId());
			if (rvfourBirthMtlb != null) {
				ur.setPartner2Firstname(rvfourBirthMtlb.getFirstname());
				ur.setPartner2Surname(rvfourBirthMtlb.getFourthname());
			}
		}
		int witness1RecordStatus = utils.checkVitalRecordStatus(mtlb.getWitness1Id(), migrationState);
		if (witness1RecordStatus == Utils.UNKNOWN || witness1RecordStatus == Utils.FOUND_BUT_VITAL_RECORD_NOT_CREATED_YET)
			return;
		else if (witness1RecordStatus == Utils.CONFIRMED_MISSING) {
			UnionRecordTag tag = null;
			tag = new UnionRecordTag();
			tag.setUnionRecordId(mtlb.getId());
			tag.setContent(CustomErrorCodes.WITNESS1_RECORD_ID_MISSING);
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
		}
		else if (witness1RecordStatus == Utils.FOUND_AS_FAMILY_TREE_RECORD) {
			UnionRecordTag tag = null;
			tag = new UnionRecordTag();
			tag.setUnionRecordId(mtlb.getId());
			tag.setKey(CustomErrorCodes.WITNESS1_RECORD_FOUND_IN_FAMILY_TREE);
			tag.setContent(mtlb.getWitness1Id().toString());
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
		}
		int witness2RecordStatus = utils.checkVitalRecordStatus(mtlb.getWitness2Id(), migrationState);
		if (witness2RecordStatus == Utils.UNKNOWN || witness2RecordStatus == Utils.FOUND_BUT_VITAL_RECORD_NOT_CREATED_YET)
			return;
		else if (witness2RecordStatus == Utils.CONFIRMED_MISSING) {
			UnionRecordTag tag = null;
			tag = new UnionRecordTag();
			tag.setUnionRecordId(mtlb.getId());
			tag.setContent(CustomErrorCodes.WITNESS2_RECORD_ID_MISSING);
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
		}
		else if (witness2RecordStatus == Utils.FOUND_AS_FAMILY_TREE_RECORD) {
			UnionRecordTag tag = null;
			tag = new UnionRecordTag();
			tag.setUnionRecordId(mtlb.getId());
			tag.setKey(CustomErrorCodes.WITNESS2_RECORD_FOUND_IN_FAMILY_TREE);
			tag.setContent(mtlb.getWitness2Id().toString());
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
		}
		ur.setId(mtlb.getId());
		ur.setAddress(mtlb.getMarriageLocation());
		ur.setPlace(mtlb.getMarriagePlace());
		ur.setPartner1VitalRecordId(mtlb.getHusbandId());
		ur.setPartner2VitalRecordId(mtlb.getWifeId());
		ur.setDateOfMarriage(mtlb.getDateOfMarriage());
		ur.setFirstWitnessId(mtlb.getWitness1Id());
		ur.setSecondWitnessId(mtlb.getWitness2Id());
		ur.setRites(mtlb.getMarriageRites());
		ur.setTimeOfMarriage(mtlb.getTimeOfMarriage());
		if (hvr != null && wvr != null) {
			if (hvr.getGender() == 2 && wvr.getGender() == 1) {
				ur.setPartner1VitalRecordId(wvr.getId());
				ur.setPartner2VitalRecordId(hvr.getId());
			} else if (hvr.getGender() == wvr.getGender()) {
				UnionRecordTag urTag = new UnionRecordTag();
				urTag.setUnionRecordId(mtlb.getId());
				urTag.setContent(hvr.getId() + " : " + wvr.getId());
				urTag.setType(Utils.ERROR_TAG_TYPE);
				urTag.setKey(CustomErrorCodes.SAME_SEX_MARRIAGE);
				tags.add(urTag);
			}
		}
		ArchideMtlbAnnex annex = null;
		try {
			annex = ArchideMtlbAnnexUtil.deserialize(mtlb.getAnnex());
		} catch (JsonProcessingException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		if (annex.getTags() != null)
			for (ArchideMtlbAnnexTag tag : annex.getTags()) {
				UnionRecordTag urTag = new UnionRecordTag();
				urTag.setUnionRecordId(mtlb.getId());
				urTag.setContent(tag.getValue());
				urTag.setType(tag.getType());
				urTag.setKey(tag.getName());
				tags.add(urTag);
			}
		ArrayList<UnionRecordMention> mentions = new ArrayList<>();
		for (ArchideMtlbAnnexLogEntry mention : annex.getMentions()) {
			UnionRecordMention vm = new UnionRecordMention();
			vm.setAddedBy(mention.getAuthor());
			vm.setContent(mention.getContent());
			vm.setTimestamp(mention.getTimestamp());
			vm.setUnionRecordId(mtlb.getId());
			mentions.add(vm);
		}
		try {
			unionRecordRepository.insert(ur);
			for (UnionRecordMention mention : mentions)
				unionRecordMentionRepository.insert(mention);
			for (UnionRecordTag tag : tags)
				unionRecordTagRepository.insert(tag);
			mtlb.setMtlbStatus(MtlbStatus.ARCHIVED);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
