package systems.archide.core.workflows.utils;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import systems.archide.core.ErrorCodes;
import systems.archide.core.pojo.RvfourBirthMtlb;
import systems.archide.core.pojo.RvfourBirthMtlbTag;
import systems.archide.core.pojo.VitalRecord;
import systems.archide.core.repository.IdcoreCustomQueries;
import systems.archide.core.repository.RvfourBirthMtlbTagRepository;
import systems.archide.core.repository.VitalRecordRepository;

@Component
public class RvFourBirthMtlbUtils {
	
	final String ERROR_TAG_TYPE = "ERROR";
	
	@Autowired
	VitalRecordRepository vitalRecordRepository;
	
	@Autowired
	IdcoreCustomQueries customQueries;
	
	@Autowired
	RvfourBirthMtlbTagRepository rvfourBirthMtlbTagRepository;
	
	public boolean validateBiographic(RvfourBirthMtlb mtlb) {
		boolean validName = validateName(mtlb);
		boolean validGenderAndBirthDate = validateGenderAndBirthDate(mtlb);
		return validName && validGenderAndBirthDate;
	}
	

	private boolean validateGenderAndBirthDate(RvfourBirthMtlb mtlb) {
		boolean valid = true;
		if (mtlb.getGender() == null) {
			createErrorTag(ErrorCodes.GENDER_MISSING, mtlb.getId());
			valid = false;
		}
		if (mtlb.getBirthdate() == null) {
			createErrorTag(ErrorCodes.BIRTH_DATE_MISSING, mtlb.getId());
			valid = false;
		}
		return valid;
	}

	private boolean validateName(RvfourBirthMtlb mtlb) {
		boolean valid = true;
		if (StringUtils.isAllBlank(mtlb.getFirstname())) {
			createErrorTag(ErrorCodes.FIRST_NAME_MISSING, mtlb.getId());
			valid = false;
		}
		if (StringUtils.isAllBlank(mtlb.getFourthname())) {
			createErrorTag(ErrorCodes.FAMILY_NAME_MISSING, mtlb.getId());
			valid = false;
		}
		String match = "^[aA-zZ�-�]{4,14}$";
		if (!StringUtils.isAllBlank(mtlb.getFirstname()))
			if (!mtlb.getFirstname().matches(match)) {
				createErrorTag(ErrorCodes.FIRST_NAME_CHAR_RULES, mtlb.getId());
				valid = false;
			}
		if (!StringUtils.isAllBlank(mtlb.getSecondname()))
			if (!mtlb.getSecondname().matches(match)) {
				createErrorTag(ErrorCodes.SECOND_NAME_CHAR_RULES, mtlb.getId());
				valid = false;
			}
		if (!StringUtils.isAllBlank(mtlb.getThirdname()))
			if (!mtlb.getThirdname().matches(match)) {
				createErrorTag(ErrorCodes.THIRD_NAME_CHAR_RULES, mtlb.getId());
				valid = false;
			}
		if (!StringUtils.isAllBlank(mtlb.getFourthname()))
			if (!mtlb.getFourthname().matches(match)) {
				createErrorTag(ErrorCodes.FAMILY_NAME_CHAR_RULES, mtlb.getId());
				valid = false;
			}
		if (!StringUtils.isAllBlank(mtlb.getFifthname()))
			if (!mtlb.getFifthname().matches(match)) {
				createErrorTag(ErrorCodes.MARRIAGE_NAME_CHAR_RULES, mtlb.getId());
				valid = false;
			}
		return valid;
	}

	public void createErrorTag(String errorCode, Long mtlbId) {
		RvfourBirthMtlbTag tag = new RvfourBirthMtlbTag();
		tag.setContent(errorCode);
		tag.setType(ERROR_TAG_TYPE);
		tag.setRvfourBirthMtlbId(mtlbId);
		rvfourBirthMtlbTagRepository.insert(tag);
	}

	public void createErrorTag(String errorCode, Long mtlbId, String type) {
		RvfourBirthMtlbTag tag = new RvfourBirthMtlbTag();
		tag.setContent(errorCode);
		tag.setType(type);
		tag.setRvfourBirthMtlbId(mtlbId);
		rvfourBirthMtlbTagRepository.insert(tag);
	}
}
