package systems.archide.core.workflows.utils;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import systems.archide.core.pojo.GlobalState;
import systems.archide.core.pojo.MtlbType;
import systems.archide.core.pojo.RvfourBirthMtlb;
import systems.archide.core.pojo.RvfourMarriageMtlb;
import systems.archide.core.pojo.UnionRecord;
import systems.archide.core.pojo.VitalRecord;
import systems.archide.core.repository.RvfourBirthMtlbRepository;
import systems.archide.core.repository.RvfourMarriageMtlbRepository;
import systems.archide.core.repository.UnionRecordRepository;
import systems.archide.core.repository.VitalRecordRepository;

@Component
public class Utils {

	public final static String INFO_TAG_TYPE = "INFO";
	public final static String WARNING_TAG_TYPE = "WARNING";
	public final static String ERROR_TAG_TYPE = "ERROR";

	public final static int FOUND = 0;
	public final static int UNKNOWN = 1;
	public final static int CONFIRMED_MISSING = 2;
	public final static int FOUND_AS_FAMILY_TREE_RECORD = 3;
	public final static int FOUND_BUT_VITAL_RECORD_NOT_CREATED_YET = 4;
	public static final int FOUND_BUT_UNION_RECORD_NOT_CREATED_YET = 5;

	@Autowired
	VitalRecordRepository vitalRecordRepository;

	@Autowired
	RvfourBirthMtlbRepository rvfourBirthMtlbRepository;
	
	@Autowired
	RvfourMarriageMtlbRepository rvfourMarriageMtlbRepository;
	
	@Autowired
	UnionRecordRepository unionRecordRepository;

	public int checkVitalRecordStatus(Long id, GlobalState migrationState) {
		VitalRecord vitalRecord = vitalRecordRepository.findById(id);
		if (vitalRecord != null)
			return FOUND;
		RvfourBirthMtlb mtlb = rvfourBirthMtlbRepository.findByVitalRecordId(id);
		if (mtlb == null) {
			if (!"FINISHED".equals(migrationState.getValue()))
				return UNKNOWN;
			else
				return CONFIRMED_MISSING;
		} else {
			if (mtlb.getMtlbType() == MtlbType.RV4_BIRTH_VITAL_RECORD)
				return FOUND_BUT_VITAL_RECORD_NOT_CREATED_YET;
			else
				return FOUND_AS_FAMILY_TREE_RECORD;
		}
	}
	
	public int checkUnionRecordStatus(Long id, GlobalState migrationState) {
		UnionRecord unionRecord = unionRecordRepository.findById(id);
		if (unionRecord != null)
			return FOUND;
		RvfourMarriageMtlb mtlb = rvfourMarriageMtlbRepository.findById(id);
		if (mtlb == null) {
			if (!"FINISHED".equals(migrationState.getValue()))
				return UNKNOWN;
			else
				return CONFIRMED_MISSING;
		} else {
			if (mtlb.getMtlbType() == MtlbType.RV4_UNION_RECORD)
				return FOUND_BUT_UNION_RECORD_NOT_CREATED_YET;
			else
				return CONFIRMED_MISSING;
		}
	}

}
